@@ -2,7 +2,7 @@
 
    client.h for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -35,7 +35,6 @@ struct _ObSessionState;
 
 typedef struct _ObClient      ObClient;
 typedef struct _ObClientIcon  ObClientIcon;
-typedef struct _ObAppSettings ObAppSettings;
 
 /* The value in client.transient_for indicating it is a transient for its
    group instead of for a single window */
@@ -71,13 +70,15 @@ typedef enum
     OB_CLIENT_FUNC_MAXIMIZE   = 1 << 3, /*!< Allow to be maximized */
     OB_CLIENT_FUNC_SHADE      = 1 << 4, /*!< Allow to be shaded */
     OB_CLIENT_FUNC_FULLSCREEN = 1 << 5, /*!< Allow to be made fullscreen */
-    OB_CLIENT_FUNC_CLOSE      = 1 << 6  /*!< Allow to be closed */
+    OB_CLIENT_FUNC_CLOSE      = 1 << 6, /*!< Allow to be closed */
+    OB_CLIENT_FUNC_ABOVE      = 1 << 7, /*!< Allow to be put in lower layer */
+    OB_CLIENT_FUNC_BELOW      = 1 << 8, /*!< Allow to be put in higher layer */
+    OB_CLIENT_FUNC_UNDECORATE = 1 << 9  /*!< Allow to be undecorated */
 } ObFunctions;
 
 struct _ObClient
 {
     ObWindow obwin;
-
     Window  window;
 
     /*! The window's decorations. NULL while the window is being managed! */
@@ -114,11 +115,12 @@ struct _ObClient
 
     /*! Normal window title */
     gchar *title;
-    /*! The count for the title. When another window with the same title
-      exists, a count will be appended to it. */
-    guint title_count;
     /*! Window title when iconified */
     gchar *icon_title;
+    /*! Hostname of machine running the client */
+    gchar *client_machine;
+    /*! The command used to run the program. Pre-XSMP window identification. */
+    gchar *wm_command;
 
     /*! The application that created the window */
     gchar *name;
@@ -139,6 +141,9 @@ struct _ObClient
     */
     Rect area;
 
+    /*! Position of the client window relative to the root window */
+    Point root_pos;
+
     /*! Position and size of the window prior to being maximized */
     Rect pre_max_area;
     /*! Position and size of the window prior to being fullscreened */
@@ -162,7 +167,7 @@ struct _ObClient
       The window manager will set this to 0 while the window is being managed,
       but needs to restore it afterwards, so it is saved here.
     */
-    guint border_width;
+    gint border_width;
 
     /*! The minimum aspect ratio the client window can be sized to.
       A value of 0 means this is ignored.
@@ -191,6 +196,9 @@ struct _ObClient
 
     /*! Window decoration and functionality hints */
     ObMwmHints mwmhints;
+
+    /*! The client's specified colormap */
+    Colormap colormap;
   
     /*! Where to place the decorated window in relation to the undecorated
       window */
@@ -208,14 +216,25 @@ struct _ObClient
       we only force it if it tries to go completely offscreen, if neither, we
       should place the window ourselves when it first appears */
     guint positioned;
+
+    /*! Was the window's size requested by the application or the user?
+      If by the application we don't let it go outside the available area */
+    guint sized;
   
     /*! Can the window receive input focus? */
     gboolean can_focus;
-    /*! Urgency flag */
-    gboolean urgent;
     /*! Notify the window when it receives focus? */
     gboolean focus_notify;
 
+#ifdef SYNC
+    /*! The client wants to sync during resizes */
+    gboolean sync_request;
+    /*! The XSync counter used for synchronizing during resizes */
+    guint32 sync_counter;
+    /*! The value we're waiting for the counter to reach */
+    gulong sync_counter_value;
+#endif
+
     /*! The window uses shape extension to be non-rectangular? */
     gboolean shaped;
 
@@ -243,6 +262,11 @@ struct _ObClient
     /*! The window should be underneath other windows of the same type.
       above takes priority over below. */
     gboolean below;
+    /*! Demands attention flag */
+    gboolean demands_attention;
+
+    /*! The urgent flag */
+    gboolean urgent;
 
     /*! The layer in which the window will be stacked, windows in lower layers
       are always below windows in higher layers. */
@@ -269,53 +293,48 @@ struct _ObClient
     ObClientIcon *icons;
     /*! The number of icons in icons */
     guint nicons;
+
+    /*! Where the window should iconify to/from */
+    Rect icon_geometry;
+
+    /*! The time when the client last received user interaction */
+    guint32 user_time;
+    /*! A separate window for the client to update it's user_time on */
+    Window  user_time_window;
 };
 
-struct _ObAppSettings
-{
-    gchar *class;
-    gchar *name;
-    gchar *role;
-
-    Point position;
-    gboolean center_x;
-    gboolean center_y;
-    gboolean pos_given;
-
-    guint desktop;
-    gint shade;
-    gint decor;
-    gint focus;
-    gint head;
-    gint iconic;
-    gint skip_pager;
-    gint skip_taskbar;
-    gint max_horz;
-    gint max_vert;
-    gint fullscreen;
-
-    gint layer;
-};
-
-extern GList *client_list;
+extern GList      *client_list;
+extern GHashTable *client_user_time_window_map;
 
 void client_startup(gboolean reconfig);
 void client_shutdown(gboolean reconfig);
 
-typedef void (*ObClientDestructor)(ObClient *client, gpointer data);
+typedef void (*ObClientCallback)(ObClient *client, gpointer data);
 
-void client_add_destructor(ObClientDestructor func, gpointer data);
-void client_remove_destructor(ObClientDestructor func);
+/* Callback functions */
+
+/*! Get notified when the client is unmanaged */
+void client_add_destroy_notify(ObClientCallback func, gpointer data);
+void client_remove_destroy_notify(ObClientCallback func);
 
 /*! Manages all existing windows */
 void client_manage_all();
-/*! Manages a given window */
+/*! Manages a given window
+*/
 void client_manage(Window win);
 /*! Unmanages all managed windows */
 void client_unmanage_all();
 /*! Unmanages a given client */
 void client_unmanage(ObClient *client);
 
+/*! This manages a window only so far as is needed to get it's decorations.
+   This is used when you want to determine a window's decorations before it
+   is mapped. Call client_fake_unmanage() with the returned client when you
+   are done with it. */
+ObClient *client_fake_manage(Window win);
+/*! Free the stuff created by client_fake_manage() */
+void client_fake_unmanage(ObClient *self);
+
 /*! Sets the client list on the root window from the client_list */
 void client_set_list();
 
@@ -329,25 +348,75 @@ gboolean client_should_show(ObClient *self);
   to them in a number of places regarding focus or user interaction. */
 gboolean client_normal(ObClient *self);
 
+/*! Returns if the window is one of an application's helper windows
+  (utilty, menu, etc) */
+gboolean client_helper(ObClient *self);
+
+/*! Return if the client is a type which should be given focus from mouse
+  presses on the *client* window. This doesn't affect clicking on the
+  decorations. This doesn't count for focus cycling, different rules apply to
+  that. */
+gboolean client_mouse_focusable(ObClient *self);
+
+/*! Return if the client is a type which should be given focus from the
+  mouse entering the window. This doesn't count for focus cycling, different
+  rules apply to that. */
+gboolean client_enter_focusable(ObClient *self);
+
 /* Returns if the window is focused */
 gboolean client_focused(ObClient *self);
 
-#define client_move(self, x, y) \
-  client_configure(self, OB_CORNER_TOPLEFT, x, y, \
-                   self->area.width, self->area.height, \
-                   TRUE, TRUE)
-#define client_resize(self, w, h) \
-  client_configure(self, OB_CORNER_TOPLEFT, self->area.x, self->area.y, \
-                   w, h, TRUE, TRUE)
-#define client_move_resize(self, x, y, w, h) \
-  client_configure(self, OB_CORNER_TOPLEFT, x, y, w, h, TRUE, TRUE)
+/*! When the client is resized but not moved, figure out the new position
+  for it based on its gravity:
+  http://standards.freedesktop.org/wm-spec/wm-spec-1.4.html#id2512541
+*/
+void client_gravity_resize_w(ObClient *self, gint *x, gint oldw, gint neww);
 
-#define client_configure(self, anchor, x, y, w, h, user, final) \
-  client_configure_full(self, anchor, x, y, w, h, user, final, FALSE)
+/*! When the client is resized but not moved, figure out the new position
+  for it based on its gravity:
+  http://standards.freedesktop.org/wm-spec/wm-spec-1.4.html#id2512541
+*/
+void client_gravity_resize_h(ObClient *self, gint *y, gint oldh, gint newh);
+
+/*! Convert a position/size from a given gravity to the client's true gravity,
+  when the client is only resizing (the reference point doesn't move)
+ */
+void client_convert_gravity_resize(ObClient *self, gint gravity,
+                                   gint *x, gint *y,
+                                   gint w, gint h);
+
+#define client_move(self, x, y) \
+  client_configure(self, x, y, self->area.width, self->area.height, TRUE, TRUE)
+#define client_resize(self, w, h) \
+  client_configure(self, self->area.x, self->area.y, w, h, TRUE, TRUE)
+#define client_move_resize(self, x, y, w, h) \
+  client_configure(self, x, y, w, h, TRUE, TRUE)
+
+/*! Figure out where a window will end up and what size it will be if you
+  told it to move/resize to these coordinates.
+
+  These values are what client_configure_full will give the window.
+
+  @param x The x coordiante of the new position for the client.
+  @param y The y coordiante of the new position for the client.
+  @param w The width component of the new size for the client.
+  @param h The height component of the new size for the client.
+  @param logicalw Returns the width component of the new logical width.
+                  This value is only returned when the new w or h calculated
+                  differ from the ones passed in.
+  @param logicalh Returns the height component of the new logical height.
+                  This value is only returned when the new w or h calculated
+                  differ from the ones passed in.
+  @param user Specifies whether this is a user-requested change or a
+              program requested change. For program requested changes, the
+              constraints are not checked.
+*/
+void client_try_configure(ObClient *self, gint *x, gint *y, gint *w, gint *h,
+                          gint *logicalw, gint *logicalh,
+                          gboolean user);
 
 /*! Move and/or resize the window.
   This also maintains things like the client's minsize, and size increments.
-  @param anchor The corner to keep in the same position when resizing.
   @param x The x coordiante of the new position for the client.
   @param y The y coordiante of the new position for the client.
   @param w The width component of the new size for the client.
@@ -362,10 +431,8 @@ gboolean client_focused(ObClient *self);
   @param force_reply Send a ConfigureNotify to the client regardless of if
                      the position changed.
 */
-void client_configure_full(ObClient *self, ObCorner anchor,
-                           gint x, gint y, gint w, gint h,
-                           gboolean user, gboolean final,
-                           gboolean force_reply);
+void client_configure(ObClient *self, gint x, gint y, gint w, gint h,
+                      gboolean user, gboolean final);
 
 void client_reconfigure(ObClient *self);
 
@@ -395,12 +462,8 @@ void client_move_onscreen(ObClient *self, gboolean rude);
 /*! Fullscreen's or unfullscreen's the client window
   @param fs true if the window should be made fullscreen; false if it should
             be returned to normal state.
-  @param savearea true to have the client's current size and position saved;
-                  otherwise, they are not. You should not save when mapping a
-                  new window that is set to fullscreen. This has no effect
-                  when restoring a window from fullscreen.
 */
-void client_fullscreen(ObClient *self, gboolean fs, gboolean savearea);
+void client_fullscreen(ObClient *self, gboolean fs);
 
 /*! Iconifies or uniconifies the client window
   @param iconic true if the window should be iconified; false if it should be
@@ -409,19 +472,15 @@ void client_fullscreen(ObClient *self, gboolean fs, gboolean savearea);
                  be uniconified to the current viewable desktop (true) or to
                  its previous desktop (false)
 */
-void client_iconify(ObClient *self, gboolean iconic, gboolean curdesk);
+void client_iconify(ObClient *self, gboolean iconic, gboolean curdesk,
+                    gboolean hide_animation);
 
 /*! Maximize or unmaximize the client window
   @param max true if the window should be maximized; false if it should be
              returned to normal size.
   @param dir 0 to set both horz and vert, 1 to set horz, 2 to set vert.
-  @param savearea true to have the client's current size and position saved;
-                  otherwise, they are not. You should not save when mapping a
-                  new window that is set to fullscreen. This has no effect
-                  when unmaximizing a window.
 */
-void client_maximize(ObClient *self, gboolean max, gint dir,
-                     gboolean savearea);
+void client_maximize(ObClient *self, gboolean max, gint dir);
 
 /*! Shades or unshades the client window
   @param shade true if the window should be shaded; false if it should be
@@ -429,6 +488,12 @@ void client_maximize(ObClient *self, gboolean max, gint dir,
 */
 void client_shade(ObClient *self, gboolean shade);
 
+/*! Set a client window to have decorations or not */
+void client_set_undecorated(ObClient *self, gboolean undecorated);
+
+/*! Hilite the window to make the user notice it */
+void client_hilite(ObClient *self, gboolean hilite);
+
 /*! Request the client to close its window */
 void client_close(ObClient *self);
 
@@ -437,9 +502,21 @@ void client_kill(ObClient *self);
 
 /*! Sends the window to the specified desktop
   @param donthide If TRUE, the window will not be shown/hidden after its
-         desktop has been changed. Generally this should be FALSE. */
+                  desktop has been changed. Generally this should be FALSE.
+*/
 void client_set_desktop(ObClient *self, guint target, gboolean donthide);
 
+/*! Show the client if it should be shown. Returns if the window is shown. */
+gboolean client_show(ObClient *self);
+
+/*! Show the client if it should be shown. Returns if the window is hidden. */
+gboolean client_hide(ObClient *self);
+
+/*! Show the client if it should be shown, and hide it if it should be
+  hidden. This is for example, when switching desktops.
+*/
+void client_showhide(ObClient *self);
+
 /*! Validate client, by making sure no Destroy or Unmap events exist in
   the event queue for the window.
   @return true if the client is valid; false if the client has already
@@ -470,44 +547,36 @@ gboolean client_can_focus(ObClient *self);
 /*! Attempt to focus the client window */
 gboolean client_focus(ObClient *self);
 
-/*! Remove focus from the client window */
-void client_unfocus(ObClient *self);
-
 /*! Activates the client for use, focusing, uniconifying it, etc. To be used
   when the user deliberately selects a window for use.
   @param here If true, then the client is brought to the current desktop;
               otherwise, the desktop is changed to where the client lives.
+  @param user If true, then a user action is what requested the activation;
+              otherwise, it means an application requested it on its own
 */
-void client_activate(ObClient *self, gboolean here);
+void client_activate(ObClient *self, gboolean here, gboolean user);
+
+/*! Bring all of its helper windows to its desktop. These are the utility and
+  stuff windows. */
+void client_bring_helper_windows(ObClient *self);
+
+/*! Bring all of its modal windows to its desktop. */
+void client_bring_modal_windows(ObClient *self);
 
 /*! Calculates the stacking layer for the client window */
 void client_calc_layer(ObClient *self);
 
-/*! Raises the client to the top of its stacking layer
-  Normally actions call to the client_* functions to make stuff go, but this
-  one is an exception. It just fires off an action, which will be queued.
-  This is because stacking order rules can be changed by focus state, and so
-  any time focus changes you have to wait for it to complete before you can
-  properly restart windows. As such, this only queues an action for later
-  execution, once the focus change has gone through.
-*/
-void client_raise(ObClient *self);
-
-/*! Lowers the client to the bottom of its stacking layer
-  Normally actions call to the client_* functions to make stuff go, but this
-  one is an exception. It just fires off an action, which will be queued.
-  This is because stacking order rules can be changed by focus state, and so
-  any time focus changes you have to wait for it to complete before you can
-  properly restart windows. As such, this only queues an action for later
-  execution, once the focus change has gone through.
-*/
-void client_lower(ObClient *self);
-
 /*! Updates the window's transient status, and any parents of it */
 void client_update_transient_for(ObClient *self);
 /*! Update the protocols that the window supports and adjusts things if they
   change */
 void client_update_protocols(ObClient *self);
+#ifdef SYNC
+/*! Updates the window's sync request counter for resizes */
+void client_update_sync_request_counter(ObClient *self);
+#endif
+/*! Updates the window's colormap */
+void client_update_colormap(ObClient *self, Colormap colormap);
 /*! Updates the WMNormalHints and adjusts things if they change */
 void client_update_normal_hints(ObClient *self);
 
@@ -519,24 +588,35 @@ void client_update_normal_hints(ObClient *self);
 void client_update_wmhints(ObClient *self);
 /*! Updates the window's title and icon title */
 void client_update_title(ObClient *self);
-/*! Updates the window's application name and class */
-void client_update_class(ObClient *self);
 /*! Updates the strut for the client */
 void client_update_strut(ObClient *self);
 /*! Updates the window's icons */
 void client_update_icons(ObClient *self);
+/*! Updates the window's user time */
+void client_update_user_time(ObClient *self);
+/*! Updates the window's user time window */
+void client_update_user_time_window(ObClient *self);
+/*! Updates the window's icon geometry (where to iconify to/from) */
+void client_update_icon_geometry(ObClient *self);
 
 /*! Set up what decor should be shown on the window and what functions should
   be allowed (ObClient::decorations and ObClient::functions).
   This also updates the NET_WM_ALLOWED_ACTIONS hint.
+  @param reconfig When TRUE, the window will be reconfigured to show the
+         changes
 */
-void client_setup_decor_and_functions(ObClient *self);
+void client_setup_decor_and_functions(ObClient *self, gboolean reconfig);
 
-/*! Retrieves the window's type and sets ObClient->type */
-void client_get_type(ObClient *self);
+/*! Sets the window's type and transient flag */
+void client_get_type_and_transientness(ObClient *self);
 
 const ObClientIcon *client_icon(ObClient *self, gint w, gint h);
 
+/*! Return TRUE if the client is transient for some other window. Return
+  FALSE if it's not transient or there is no window for it to be
+  transient for */
+gboolean client_has_parent(ObClient *self);
+
 /*! Searches a client's direct parents for a focused window. The function does
   not check for the passed client, only for *ONE LEVEL* of its parents.
   If no focused parentt is found, NULL is returned.
@@ -561,7 +641,26 @@ ObClient *client_search_focus_tree_full(ObClient *self);
 */
 ObClient *client_search_modal_child(ObClient *self);
 
-ObClient *client_search_top_transient(ObClient *self);
+/*! Returns a list of top-level windows which this is a transient for.
+  It will only contain more than 1 element if the client is transient for its
+  group.
+*/
+GSList *client_search_all_top_parents(ObClient *self);
+
+/*! Returns a list of top-level windows which this is a transient for, and
+  which are in the same layer as this client.
+  It will only contain more than 1 element if the client is transient for its
+  group.
+*/
+GSList *client_search_all_top_parents_layer(ObClient *self);
+
+/*! Returns a window's top level parent. This only counts direct parents,
+  not groups if it is transient for its group.
+*/
+ObClient *client_search_top_normal_parent(ObClient *self);
+
+/*! Is one client a direct child of another (i.e. not through the group.) */
+gboolean client_is_direct_child(ObClient *parent, ObClient *child);
 
 /*! Search for a parent of a client. This only searches up *ONE LEVEL*, and
   returns the searched for parent if it is a parent, or NULL if not. */
@@ -571,26 +670,18 @@ ObClient *client_search_parent(ObClient *self, ObClient *search);
   NULL is returned if the given search is not a transient of the client. */
 ObClient *client_search_transient(ObClient *self, ObClient *search);
 
-/*! Return the "closest" client in the given direction */
-ObClient *client_find_directional(ObClient *c, ObDirection dir);
-
 /*! Return the closest edge in the given direction */
-gint client_directional_edge_search(ObClient *c, ObDirection dir);
+gint client_directional_edge_search(ObClient *c, ObDirection dir, gboolean hang);
 
 /*! Set a client window to be above/below other clients.
-  @layer < 0 indicates the client should be placed below other clients.<br>
-         = 0 indicates the client should be placed with other clients.<br>
+  @layer < 0 indicates the client should be placed below other clients.<br />
+         = 0 indicates the client should be placed with other clients.<br />
          > 0 indicates the client should be placed above other clients.
 */
 void client_set_layer(ObClient *self, gint layer);
 
-/*! Set a client window to have decorations or not */
-void client_set_undecorated(ObClient *self, gboolean undecorated);
-
 guint client_monitor(ObClient *self);
 
-void client_update_sm_client_id(ObClient *self);
-
 ObClient* client_under_pointer();
 
 gboolean client_has_group_siblings(ObClient *self);
