@@ -2,7 +2,7 @@
 
    dock.c for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -57,8 +57,9 @@ void dock_startup(gboolean reconfig)
         GList *it;
 
         XSetWindowBorder(ob_display, dock->frame,
-                         RrColorPixel(ob_rr_theme->b_color));
-        XSetWindowBorderWidth(ob_display, dock->frame, ob_rr_theme->bwidth);
+                         RrColorPixel(ob_rr_theme->
+                                      frame_focused_border_color));
+        XSetWindowBorderWidth(ob_display, dock->frame, ob_rr_theme->fbwidth);
 
         RrAppearanceFree(dock->a_frame);
         dock->a_frame = RrAppearanceCopy(ob_rr_theme->a_focused_title);
@@ -91,8 +92,8 @@ void dock_startup(gboolean reconfig)
                                 &attrib);
     dock->a_frame = RrAppearanceCopy(ob_rr_theme->a_focused_title);
     XSetWindowBorder(ob_display, dock->frame,
-                     RrColorPixel(ob_rr_theme->b_color));
-    XSetWindowBorderWidth(ob_display, dock->frame, ob_rr_theme->bwidth);
+                     RrColorPixel(ob_rr_theme->frame_focused_border_color));
+    XSetWindowBorderWidth(ob_display, dock->frame, ob_rr_theme->fbwidth);
 
     g_hash_table_insert(window_map, &dock->frame, dock);
     stacking_add(DOCK_AS_WINDOW(dock));
@@ -220,7 +221,7 @@ void dock_configure()
     gint strw, strh;
     Rect *a;
 
-    RrMinsize(dock->a_frame, &minw, &minh);
+    RrMinSize(dock->a_frame, &minw, &minh);
 
     dock->w = dock->h = 0;
 
@@ -261,8 +262,8 @@ void dock_configure()
     }
 
     /* used for calculating offsets */
-    dock->w += ob_rr_theme->bwidth * 2;
-    dock->h += ob_rr_theme->bwidth * 2;
+    dock->w += ob_rr_theme->fbwidth * 2;
+    dock->h += ob_rr_theme->fbwidth * 2;
 
     a = screen_physical_area();
 
@@ -313,6 +314,8 @@ void dock_configure()
             dock->y = a->height;
             gravity = SouthEastGravity;
             break;
+        default:
+            g_assert_not_reached();
         }
     }
 
@@ -347,51 +350,51 @@ void dock_configure()
             case OB_DIRECTION_NORTHWEST:
                 switch (config_dock_orient) {
                 case OB_ORIENTATION_HORZ:
-                    dock->y -= dock->h - ob_rr_theme->bwidth;
+                    dock->y -= dock->h - ob_rr_theme->fbwidth;
                     break;
                 case OB_ORIENTATION_VERT:
-                    dock->x -= dock->w - ob_rr_theme->bwidth;
+                    dock->x -= dock->w - ob_rr_theme->fbwidth;
                     break;
                 }
                 break;
             case OB_DIRECTION_NORTH:
-                dock->y -= dock->h - ob_rr_theme->bwidth;
+                dock->y -= dock->h - ob_rr_theme->fbwidth;
                 break;
             case OB_DIRECTION_NORTHEAST:
                 switch (config_dock_orient) {
                 case OB_ORIENTATION_HORZ:
-                    dock->y -= dock->h - ob_rr_theme->bwidth;
+                    dock->y -= dock->h - ob_rr_theme->fbwidth;
                     break;
                 case OB_ORIENTATION_VERT:
-                    dock->x += dock->w - ob_rr_theme->bwidth;
+                    dock->x += dock->w - ob_rr_theme->fbwidth;
                     break;
                 }
                 break;
             case OB_DIRECTION_WEST:
-                dock->x -= dock->w - ob_rr_theme->bwidth;
+                dock->x -= dock->w - ob_rr_theme->fbwidth;
                 break;
             case OB_DIRECTION_EAST:
-                dock->x += dock->w - ob_rr_theme->bwidth;
+                dock->x += dock->w - ob_rr_theme->fbwidth;
                 break;
             case OB_DIRECTION_SOUTHWEST:
                 switch (config_dock_orient) {
                 case OB_ORIENTATION_HORZ:
-                    dock->y += dock->h - ob_rr_theme->bwidth;
+                    dock->y += dock->h - ob_rr_theme->fbwidth;
                     break;
                 case OB_ORIENTATION_VERT:
-                    dock->x -= dock->w - ob_rr_theme->bwidth;
+                    dock->x -= dock->w - ob_rr_theme->fbwidth;
                     break;
                 } break;
             case OB_DIRECTION_SOUTH:
-                dock->y += dock->h - ob_rr_theme->bwidth;
+                dock->y += dock->h - ob_rr_theme->fbwidth;
                 break;
             case OB_DIRECTION_SOUTHEAST:
                 switch (config_dock_orient) {
                 case OB_ORIENTATION_HORZ:
-                    dock->y += dock->h - ob_rr_theme->bwidth;
+                    dock->y += dock->h - ob_rr_theme->fbwidth;
                     break;
                 case OB_ORIENTATION_VERT:
-                    dock->x += dock->w - ob_rr_theme->bwidth;
+                    dock->x += dock->w - ob_rr_theme->fbwidth;
                     break;
                 }
                 break;
@@ -400,8 +403,8 @@ void dock_configure()
     }
 
     if (!config_dock_floating && config_dock_hide) {
-        strw = ob_rr_theme->bwidth;
-        strh = ob_rr_theme->bwidth;
+        strw = ob_rr_theme->fbwidth;
+        strh = ob_rr_theme->fbwidth;
     } else {
         strw = dock->w;
         strh = dock->h;
@@ -411,7 +414,8 @@ void dock_configure()
     if (!dock->dock_apps) {
         STRUT_PARTIAL_SET(dock_strut, 0, 0, 0, 0,
                           0, 0, 0, 0, 0, 0, 0, 0);
-    } else if (config_dock_floating || config_dock_nostrut) {
+    } else if (config_dock_floating || config_dock_nostrut)
+    {
         STRUT_PARTIAL_SET(dock_strut, 0, 0, 0, 0,
                           0, 0, 0, 0, 0, 0, 0, 0);
     } else {
@@ -499,8 +503,8 @@ void dock_configure()
     dock->h += minh;
 
     /* not used for actually sizing shit */
-    dock->w -= ob_rr_theme->bwidth * 2;
-    dock->h -= ob_rr_theme->bwidth * 2;
+    dock->w -= ob_rr_theme->fbwidth * 2;
+    dock->h -= ob_rr_theme->fbwidth * 2;
 
     if (dock->dock_apps) {
         g_assert(dock->w > 0);
@@ -515,8 +519,8 @@ void dock_configure()
         XUnmapWindow(ob_display, dock->frame);
 
     /* but they are useful outside of this function! */
-    dock->w += ob_rr_theme->bwidth * 2;
-    dock->h += ob_rr_theme->bwidth * 2;
+    dock->w += ob_rr_theme->fbwidth * 2;
+    dock->h += ob_rr_theme->fbwidth * 2;
 
     screen_update_areas();
 }
@@ -578,6 +582,8 @@ void dock_app_drag(ObDockApp *app, XMotionEvent *e)
     case OB_ORIENTATION_VERT:
         after = (y > over->h / 2);
         break;
+    default:
+        g_assert_not_reached();
     }
 
     /* remove before doing the it->next! */
@@ -612,14 +618,14 @@ void dock_hide(gboolean hide)
     if (!hide) {
         if (dock->hidden && config_dock_hide) {
             ob_main_loop_timeout_add(ob_main_loop, config_dock_show_delay,
-                                 show_timeout, NULL, NULL);
+                                 show_timeout, NULL, g_direct_equal, NULL);
         } else if (!dock->hidden && config_dock_hide) {
             ob_main_loop_timeout_remove(ob_main_loop, hide_timeout);
         }
     } else {
         if (!dock->hidden && config_dock_hide) {
             ob_main_loop_timeout_add(ob_main_loop, config_dock_hide_delay,
-                                 hide_timeout, NULL, NULL);
+                                 hide_timeout, NULL, g_direct_equal, NULL);
         } else if (dock->hidden && config_dock_hide) {
             ob_main_loop_timeout_remove(ob_main_loop, show_timeout);
         }
