@@ -2,7 +2,7 @@
 
    extensions.h for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,6 +36,9 @@
 #ifdef    XRANDR
 #include <X11/extensions/Xrandr.h>
 #endif
+#ifdef    SYNC
+#include <X11/extensions/sync.h>
+#endif
 
 #include <glib.h>
 
@@ -59,6 +62,11 @@ extern gboolean extensions_randr;
 /*! Base for events for the Randr extension */
 extern gint extensions_randr_event_basep;
 
+/*! Does the display have the Sync extension? */
+extern gboolean extensions_sync;
+/*! Base for events for the Sync extension */
+extern gint extensions_sync_event_basep;
+
 void extensions_query_all();
 
 void extensions_xinerama_screens(Rect **areas, guint *nxin);
