@@ -2,7 +2,7 @@
 
    keyboard.c for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,84 +27,116 @@
 #include "client.h"
 #include "action.h"
 #include "prop.h"
+#include "menuframe.h"
 #include "config.h"
 #include "keytree.h"
 #include "keyboard.h"
 #include "translate.h"
 #include "moveresize.h"
+#include "popup.h"
+#include "gettext.h"
 
 #include <glib.h>
 
-KeyBindingTree *keyboard_firstnode;
-
 typedef struct {
+    gboolean active;
     guint state;
     ObClient *client;
-    GSList *actions;
-    ObFrameContext context;
+    ObAction *action;
 } ObInteractiveState;
 
-static GSList *interactive_states;
-
+KeyBindingTree *keyboard_firstnode = NULL;
+static ObPopup *popup = NULL;
+static ObInteractiveState istate;
 static KeyBindingTree *curpos;
 
-static void grab_for_window(Window win, gboolean grab)
+static void grab_keys(gboolean grab)
 {
     KeyBindingTree *p;
 
-    ungrab_all_keys(win);
+    ungrab_all_keys(RootWindow(ob_display, ob_screen));
 
     if (grab) {
         p = curpos ? curpos->first_child : keyboard_firstnode;
         while (p) {
-            grab_key(p->key, p->state, win, GrabModeAsync);
+            grab_key(p->key, p->state, RootWindow(ob_display, ob_screen),
+                     GrabModeAsync);
             p = p->next_sibling;
         }
         if (curpos)
             grab_key(config_keyboard_reset_keycode,
                      config_keyboard_reset_state,
-                     win, GrabModeAsync);
+                     RootWindow(ob_display, ob_screen), GrabModeAsync);
     }
 }
 
-void keyboard_grab_for_client(ObClient *c, gboolean grab)
-{
-    grab_for_window(c->window, grab);
-}
-
-static void grab_keys(gboolean grab)
-{
-    GList *it;
-
-    grab_for_window(screen_support_win, grab);
-    for (it = client_list; it; it = g_list_next(it))
-        grab_for_window(((ObClient*)it->data)->window, grab);
-}
-
 static gboolean chain_timeout(gpointer data)
 {
-    keyboard_reset_chains();
-
+    keyboard_reset_chains(0);
     return FALSE; /* don't repeat */
 }
 
-void keyboard_reset_chains()
+static void set_curpos(KeyBindingTree *newpos)
 {
-    ob_main_loop_timeout_remove(ob_main_loop, chain_timeout);
+    if (curpos == newpos) return;
 
-    if (curpos) {
-        grab_keys(FALSE);
-        curpos = NULL;
-        grab_keys(TRUE);
+    grab_keys(FALSE);
+    curpos = newpos;
+    grab_keys(TRUE);
+
+    if (curpos != NULL) {
+        gchar *text = NULL;
+        GList *it;
+
+        for (it = curpos->keylist; it; it = g_list_next(it)) {
+            gchar *oldtext = text;
+            if (text == NULL)
+                text = g_strdup(it->data);
+            else
+                text = g_strconcat(text, " - ", it->data, NULL);
+            g_free(oldtext);
+        }
+
+        popup_position(popup, NorthWestGravity, 10, 10);
+        /* 1 second delay for the popup to show */
+        popup_delay_show(popup, G_USEC_PER_SEC, text);
+        g_free(text);
+    } else {
+        popup_hide(popup);
     }
 }
 
+void keyboard_reset_chains(gint break_chroots)
+{
+    KeyBindingTree *p;
+
+    for (p = curpos; p; p = p->parent) {
+        if (p->chroot) {
+            if (break_chroots == 0) break; /* stop here */
+            if (break_chroots > 0)
+                --break_chroots;
+        }
+    }
+    set_curpos(p);
+}
+
 void keyboard_unbind_all()
 {
     tree_destroy(keyboard_firstnode);
     keyboard_firstnode = NULL;
-    grab_keys(FALSE);
-    curpos = NULL;
+}
+
+void keyboard_chroot(GList *keylist)
+{
+    /* try do it in the existing tree. if we can't that means it is an empty
+       chroot binding. so add it to the tree then. */
+    if (!tree_chroot(keyboard_firstnode, keylist)) {
+        KeyBindingTree *tree;
+        if (!(tree = tree_build(keylist)))
+            return;
+        tree_chroot(tree, keylist);
+        tree_assimilate(tree);
+    }
 }
 
 gboolean keyboard_bind(GList *keylist, ObAction *action)
@@ -127,7 +159,7 @@ gboolean keyboard_bind(GList *keylist, ObAction *action)
         t = tree;
 
     if (conflict) {
-        g_warning("conflict with binding");
+        g_message(_("Conflict with key binding in config file"));
         tree_destroy(tree);
         return FALSE;
     }
@@ -156,91 +188,86 @@ gboolean keyboard_bind(GList *keylist, ObAction *action)
     return TRUE;
 }
 
+static void keyboard_interactive_end(guint state, gboolean cancel, Time time,
+                                     gboolean ungrab)
+{
+    GSList *alist;
+
+    g_assert(istate.active);
+
+    /* ungrab first so they won't be NotifyWhileGrabbed */
+    if (ungrab)
+        ungrab_keyboard();
+
+    /* set this before running the actions so they know the keyboard is not
+       grabbed */
+    istate.active = FALSE;
+
+    alist = g_slist_append(NULL, istate.action);
+    action_run_interactive(alist, istate.client, state, time, cancel, TRUE);
+    g_slist_free(alist);
+}
+
+static void keyboard_interactive_end_client(ObClient *client, gpointer data)
+{
+    if (istate.active && istate.client == client)
+        istate.client = NULL;
+}
+
+
+void keyboard_interactive_cancel()
+{
+    keyboard_interactive_end(0, TRUE, event_curtime, TRUE);
+}
+
 gboolean keyboard_interactive_grab(guint state, ObClient *client,
                                    ObAction *action)
 {
-    ObInteractiveState *s;
-
     g_assert(action->data.any.interactive);
 
-    if (!interactive_states) {
-        if (!grab_keyboard(TRUE))
+    if (!istate.active) {
+        if (!grab_keyboard())
             return FALSE;
-        if (!grab_pointer(TRUE, OB_CURSOR_NONE)) {
-            grab_keyboard(FALSE);
-            return FALSE;
-        }
+    } else if (action->func != istate.action->func) {
+        keyboard_interactive_end(state, TRUE, action->data.any.time, FALSE);
     }
 
-    s = g_new(ObInteractiveState, 1);
-
-    s->state = state;
-    s->client = client;
-    s->actions = g_slist_append(NULL, action);
-
-    interactive_states = g_slist_append(interactive_states, s);
+    istate.active = TRUE;
+    istate.state = state;
+    istate.client = client;
+    istate.action = action;
 
     return TRUE;
 }
 
-void keyboard_interactive_end(ObInteractiveState *s,
-                              guint state, gboolean cancel)
-{
-    action_run_interactive(s->actions, s->client, state, cancel, TRUE);
-
-    g_slist_free(s->actions);
-    g_free(s);
-
-    interactive_states = g_slist_remove(interactive_states, s);
-
-    if (!interactive_states) {
-        grab_keyboard(FALSE);
-        grab_pointer(FALSE, OB_CURSOR_NONE);
-        keyboard_reset_chains();
-    }
-}
-
-void keyboard_interactive_end_client(ObClient *client, gpointer data)
-{
-    GSList *it, *next;
-
-    for (it = interactive_states; it; it = next) {
-        ObInteractiveState *s = it->data;
-
-        next = g_slist_next(it);
-
-        if (s->client == client)
-            s->client = NULL;
-    }
-}
-
 gboolean keyboard_process_interactive_grab(const XEvent *e, ObClient **client)
 {
-    GSList *it, *next;
     gboolean handled = FALSE;
     gboolean done = FALSE;
     gboolean cancel = FALSE;
 
-    for (it = interactive_states; it; it = next) {
-        ObInteractiveState *s = it->data;
-
-        next = g_slist_next(it);
-        
-        if ((e->type == KeyRelease && 
-             !(s->state & e->xkey.state)))
+    if (istate.active) {
+        if ((e->type == KeyRelease && !(istate.state & e->xkey.state))) {
             done = TRUE;
-        else if (e->type == KeyPress) {
-            /*if (e->xkey.keycode == ob_keycode(OB_KEY_RETURN))
-                done = TRUE;
-            else */if (e->xkey.keycode == ob_keycode(OB_KEY_ESCAPE))
-                cancel = done = TRUE;
-        }
-        if (done) {
-            keyboard_interactive_end(s, e->xkey.state, cancel);
-
             handled = TRUE;
-        } else
-            *client = s->client;
+        } else if (e->type == KeyPress) {
+            /*if (e->xkey.keycode == ob_keycode(OB_KEY_RETURN))
+              done = TRUE;
+              else */if (e->xkey.keycode == ob_keycode(OB_KEY_ESCAPE)) {
+                  cancel = done = TRUE;
+                  handled = TRUE;
+              }
+        } else if (e->type == ButtonPress) {
+            cancel = TRUE;
+            done = TRUE;
+            handled = FALSE;
+        }
+
+        if (done)
+            keyboard_interactive_end(e->xkey.state, cancel, e->xkey.time,TRUE);
+
+        if (handled)
+            *client = istate.client;
     }
 
     return handled;
@@ -255,7 +282,8 @@ void keyboard_event(ObClient *client, const XEvent *e)
     if (e->xkey.keycode == config_keyboard_reset_keycode &&
         e->xkey.state == config_keyboard_reset_state)
     {
-        keyboard_reset_chains();
+        ob_main_loop_timeout_remove(ob_main_loop, chain_timeout);
+        keyboard_reset_chains(-1);
         return;
     }
 
@@ -267,20 +295,25 @@ void keyboard_event(ObClient *client, const XEvent *e)
         if (p->key == e->xkey.keycode &&
             p->state == e->xkey.state)
         {
+            /* if we hit a key binding, then close any open menus and run it */
+            if (menu_frame_visible)
+                menu_frame_hide_all();
+
             if (p->first_child != NULL) { /* part of a chain */
                 ob_main_loop_timeout_remove(ob_main_loop, chain_timeout);
-                /* 5 second timeout for chains */
-                ob_main_loop_timeout_add(ob_main_loop, 5 * G_USEC_PER_SEC,
-                                         chain_timeout, NULL, NULL);
-                grab_keys(FALSE);
-                curpos = p;
-                grab_keys(TRUE);
-            } else {
-
-                keyboard_reset_chains();
+                /* 3 second timeout for chains */
+                ob_main_loop_timeout_add(ob_main_loop, 3 * G_USEC_PER_SEC,
+                                         chain_timeout, NULL,
+                                         g_direct_equal, NULL);
+                set_curpos(p);
+            } else if (p->chroot)         /* an empty chroot */
+                set_curpos(p);
+            else {
+                keyboard_reset_chains(0);
 
                 action_run_key(p->actions, client, e->xkey.state,
-                               e->xkey.x_root, e->xkey.y_root);
+                               e->xkey.x_root, e->xkey.y_root,
+                               e->xkey.time);
             }
             break;
         }
@@ -290,31 +323,33 @@ void keyboard_event(ObClient *client, const XEvent *e)
 
 gboolean keyboard_interactively_grabbed()
 {
-    return !!interactive_states;
+    return istate.active;
 }
 
 void keyboard_startup(gboolean reconfig)
 {
     grab_keys(TRUE);
+    popup = popup_new(FALSE);
+    popup_set_text_align(popup, RR_JUSTIFY_CENTER);
 
     if (!reconfig)
-        client_add_destructor(keyboard_interactive_end_client, NULL);
+        client_add_destroy_notify(keyboard_interactive_end_client, NULL);
 }
 
 void keyboard_shutdown(gboolean reconfig)
 {
-    GSList *it;
-
     if (!reconfig)
-        client_remove_destructor(keyboard_interactive_end_client);
+        client_remove_destroy_notify(keyboard_interactive_end_client);
 
-    for (it = interactive_states; it; it = g_slist_next(it))
-        g_free(it->data);
-    g_slist_free(interactive_states);
-    interactive_states = NULL;
+    if (istate.active)
+        keyboard_interactive_cancel();
 
     ob_main_loop_timeout_remove(ob_main_loop, chain_timeout);
 
     keyboard_unbind_all();
+    set_curpos(NULL);
+
+    popup_free(popup);
+    popup = NULL;
 }
 
