@@ -2,7 +2,7 @@
 
    mainloop.c for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include "mainloop.h"
 #include "action.h"
 #include "client.h"
+#include "event.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -98,6 +99,7 @@ struct _ObMainLoopTimer
     gulong delay;
     GSourceFunc func;
     gpointer data;
+    GEqualFunc equal;
     GDestroyNotify destroy;
 
     /* The timer needs to be freed */
@@ -280,7 +282,7 @@ void ob_main_loop_run(ObMainLoop *loop)
     loop->run = TRUE;
     loop->running = TRUE;
 
-    client_add_destructor(ob_main_loop_client_destroy, loop);
+    client_add_destroy_notify(ob_main_loop_client_destroy, loop);
 
     while (loop->run) {
         if (loop->signal_fired) {
@@ -325,14 +327,16 @@ void ob_main_loop_run(ObMainLoop *loop)
                 {
                     loop->action_queue =
                         g_slist_delete_link(loop->action_queue,
-                                loop->action_queue);
+                                            loop->action_queue);
                     action_unref(act);
                     act = NULL;
                 }
             } while (!act && loop->action_queue);
 
             if  (act) {
+                event_curtime = act->data.any.time;
                 act->func(&act->data);
+                event_curtime = CurrentTime;
                 loop->action_queue =
                     g_slist_delete_link(loop->action_queue,
                                         loop->action_queue);
@@ -361,7 +365,7 @@ void ob_main_loop_run(ObMainLoop *loop)
         }
     }
 
-    client_remove_destructor(ob_main_loop_client_destroy);
+    client_remove_destroy_notify(ob_main_loop_client_destroy);
 
     loop->running = FALSE;
 }
@@ -418,7 +422,8 @@ static void sighandler(gint sig)
             /* XXX special case for signals that default to core dump.
                but throw some helpful output here... */
 
-            fprintf(stderr, "Fuck yah. Core dump. (Signal=%d)\n", sig);
+            fprintf(stderr, "How are you gentlemen? All your base are"
+                    " belong to us. (Openbox received signal %d)\n", sig);
 
             /* die with a core dump */
             abort();
@@ -581,12 +586,14 @@ void ob_main_loop_timeout_add(ObMainLoop *loop,
                               gulong microseconds,
                               GSourceFunc handler,
                               gpointer data,
+                              GEqualFunc cmp,
                               GDestroyNotify notify)
 {
     ObMainLoopTimer *t = g_new(ObMainLoopTimer, 1);
     t->delay = microseconds;
     t->func = handler;
     t->data = data;
+    t->equal = cmp;
     t->destroy = notify;
     t->del_me = FALSE;
     g_get_current_time(&loop->now);
@@ -615,7 +622,7 @@ void ob_main_loop_timeout_remove_data(ObMainLoop *loop, GSourceFunc handler,
 
     for (it = loop->timers; it; it = g_slist_next(it)) {
         ObMainLoopTimer *t = it->data;
-        if (t->func == handler && t->data == data) {
+        if (t->func == handler && t->equal(t->data, data)) {
             t->del_me = TRUE;
             if (cancel_dest)
                 t->destroy = NULL;
