@@ -2,7 +2,7 @@
 
    menu.c for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include "debug.h"
 #include "menu.h"
 #include "openbox.h"
+#include "mainloop.h"
 #include "stacking.h"
 #include "client.h"
 #include "config.h"
@@ -30,6 +31,8 @@
 #include "misc.h"
 #include "client_menu.h"
 #include "client_list_menu.h"
+#include "client_list_combined_menu.h"
+#include "gettext.h"
 #include "parser/parse.h"
 
 typedef struct _ObMenuParseState ObMenuParseState;
@@ -43,6 +46,7 @@ struct _ObMenuParseState
 static GHashTable *menu_hash = NULL;
 static ObParseInst *menu_parse_inst;
 static ObMenuParseState menu_parse_state;
+static gboolean menu_can_hide = FALSE;
 
 static void menu_destroy_hash_value(ObMenu *self);
 static void parse_menu_item(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
@@ -52,6 +56,9 @@ static void parse_menu_separator(ObParseInst *i,
                                  gpointer data);
 static void parse_menu(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
                        gpointer data);
+static gunichar parse_shortcut(const gchar *label, gboolean allow_shortcut,
+                               gchar **strippedlabel, guint *position);
+
 
 static void client_dest(ObClient *client, gpointer data)
 {
@@ -71,6 +78,7 @@ void menu_startup(gboolean reconfig)
                                       (GDestroyNotify)menu_destroy_hash_value);
 
     client_list_menu_startup(reconfig);
+    client_list_combined_menu_startup(reconfig);
     client_menu_startup();
 
     menu_parse_inst = parse_startup();
@@ -88,30 +96,35 @@ void menu_startup(gboolean reconfig)
             loaded = TRUE;
             parse_tree(menu_parse_inst, doc, node->children);
             xmlFreeDoc(doc);
-        }
+        } else
+            g_message(_("Unable to find a valid menu file '%s'"),
+                      (const gchar*)it->data);
     }
     if (!loaded) {
         if (parse_load_menu("menu.xml", &doc, &node)) {
             parse_tree(menu_parse_inst, doc, node->children);
             xmlFreeDoc(doc);
-        }
+        } else
+            g_message(_("Unable to find a valid menu file '%s'"),
+                      "menu.xml");
     }
     
     g_assert(menu_parse_state.parent == NULL);
 
     if (!reconfig)
-        client_add_destructor(client_dest, NULL);
+        client_add_destroy_notify(client_dest, NULL);
 }
 
 void menu_shutdown(gboolean reconfig)
 {
     if (!reconfig)
-        client_remove_destructor(client_dest);
+        client_remove_destroy_notify(client_dest);
 
     parse_shutdown(menu_parse_inst);
     menu_parse_inst = NULL;
 
     client_list_menu_shutdown(reconfig);
+    client_list_combined_menu_shutdown(reconfig);
 
     menu_frame_hide_all();
     g_hash_table_destroy(menu_hash);
@@ -135,7 +148,8 @@ void menu_pipe_execute(ObMenu *self)
         return;
 
     if (!g_spawn_command_line_sync(self->execute, &output, NULL, NULL, &err)) {
-        g_warning("Failed to execute command for pipe-menu: %s", err->message);
+        g_message(_("Failed to execute command for pipe-menu '%s': %s"),
+                  self->execute, err->message);
         g_error_free(err);
         return;
     }
@@ -151,7 +165,7 @@ void menu_pipe_execute(ObMenu *self)
         parse_tree(menu_parse_inst, doc, node->children);
         xmlFreeDoc(doc);
     } else {
-        g_warning("Invalid output from pipe-menu: %s", self->execute);
+        g_message(_("Invalid output from pipe-menu '%s'"), self->execute);
     }
 
     g_free(output);
@@ -164,11 +178,65 @@ static ObMenu* menu_from_name(gchar *name)
     g_assert(name != NULL);
 
     if (!(self = g_hash_table_lookup(menu_hash, name)))
-        g_warning("Attempted to access menu '%s' but it does not exist.",
+        g_message(_("Attempted to access menu '%s' but it does not exist"),
                   name);
     return self;
 }  
 
+#define VALID_SHORTCUT(c) (((c) >= '0' && (c) <= '9') || \
+                           ((c) >= 'A' && (c) <= 'Z') || \
+                           ((c) >= 'a' && (c) <= 'z'))
+
+static gunichar parse_shortcut(const gchar *label, gboolean allow_shortcut,
+                               gchar **strippedlabel, guint *position)
+{
+    gunichar shortcut = 0;
+    
+    *position = 0;
+
+    g_assert(strippedlabel != NULL);
+
+    if (label == NULL) {
+        *strippedlabel = NULL;
+    } else {
+        gchar *i;
+
+        *strippedlabel = g_strdup(label);
+
+        /* if allow_shortcut is false, then you can't use the &, instead you
+           have to just use the first valid character
+        */
+
+        i = strchr(*strippedlabel, '&');
+        if (allow_shortcut && i != NULL) {
+            /* there is an ampersand in the string */
+
+            /* you have to use a printable ascii character for shortcuts
+               don't allow space either, so you can have like "a & b"
+            */
+            if (VALID_SHORTCUT(*(i+1))) {
+                shortcut = g_unichar_tolower(g_utf8_get_char(i+1));
+                *position = i - *strippedlabel;
+
+                /* remove the & from the string */
+                for (; *i != '\0'; ++i)
+                    *i = *(i+1);
+            }
+        } else {
+            /* there is no ampersand, so find the first valid character to use
+               instead */
+
+            for (i = *strippedlabel; *i != '\0'; ++i)
+                if (VALID_SHORTCUT(*i)) {
+                    *position = i - *strippedlabel;
+                    shortcut = g_unichar_tolower(g_utf8_get_char(i));
+                    break;
+                }
+        }
+    }
+    return shortcut;
+}
+
 static void parse_menu_item(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
                             gpointer data)
 {
@@ -186,7 +254,7 @@ static void parse_menu_item(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
                     if (a)
                         acts = g_slist_append(acts, a);
                 }
-            menu_add_normal(state->parent, -1, label, acts);
+            menu_add_normal(state->parent, -1, label, acts, FALSE);
             g_free(label);
         }
     }
@@ -198,8 +266,15 @@ static void parse_menu_separator(ObParseInst *i,
 {
     ObMenuParseState *state = data;
 
-    if (state->parent)
-        menu_add_separator(state->parent, -1);
+    if (state->parent) {
+        gchar *label;
+
+        if (!parse_attr_string("label", node, &label))
+            label = NULL;
+
+        menu_add_separator(state->parent, -1, label);
+        g_free(label);
+    }
 }
 
 static void parse_menu(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
@@ -216,7 +291,7 @@ static void parse_menu(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
         if (!parse_attr_string("label", node, &title))
             goto parse_menu_fail;
 
-        if ((menu = menu_new(name, title, NULL))) {
+        if ((menu = menu_new(name, title, FALSE, NULL))) {
             menu->pipe_creator = state->pipe_creator;
             if (parse_attr_string("execute", node, &script)) {
                 menu->execute = parse_expand_tilde(script);
@@ -240,17 +315,42 @@ parse_menu_fail:
     g_free(script);
 }
 
-ObMenu* menu_new(gchar *name, gchar *title, gpointer data)
+ObMenu* menu_new(const gchar *name, const gchar *title,
+                 gboolean allow_shortcut_selection, gpointer data)
 {
     ObMenu *self;
 
     self = g_new0(ObMenu, 1);
     self->name = g_strdup(name);
-    self->title = g_strdup(title);
     self->data = data;
 
+    self->shortcut = parse_shortcut(title, allow_shortcut_selection,
+                                    &self->title, &self->shortcut_position);
+
     g_hash_table_replace(menu_hash, self->name, self);
 
+    /* Each menu has a single more_menu.  When the menu spills past what
+       can fit on the screen, a new menu frame entry is created from this
+       more_menu, and a new menu frame for the submenu is created for this
+       menu, also pointing to the more_menu.
+
+       This can be done multiple times using the same more_menu.
+
+       more_menu->more_menu will always be NULL, since there is only 1 for
+       each menu. */
+    self->more_menu = g_new0(ObMenu, 1);
+    self->more_menu->name = _("More...");
+    self->more_menu->title = _("More...");
+    self->more_menu->data = data;
+    self->more_menu->shortcut = g_unichar_tolower(g_utf8_get_char("M"));
+
+    self->more_menu->show_func = self->show_func;
+    self->more_menu->hide_func = self->hide_func;
+    self->more_menu->update_func = self->update_func;
+    self->more_menu->execute_func = self->execute_func;
+    self->more_menu->destroy_func = self->destroy_func;
+    self->more_menu->place_func = self->place_func;
+
     return self;
 }
 
@@ -275,20 +375,27 @@ static void menu_destroy_hash_value(ObMenu *self)
     g_free(self->name);
     g_free(self->title);
     g_free(self->execute);
+    g_free(self->more_menu);
 
     g_free(self);
 }
 
 void menu_free(ObMenu *menu)
 {
-    g_hash_table_remove(menu_hash, menu->name);
+    if (menu)
+        g_hash_table_remove(menu_hash, menu->name);
 }
 
-void menu_show(gchar *name, gint x, gint y, ObClient *client)
+static gboolean menu_hide_delay_func(gpointer data)
+{
+    menu_can_hide = TRUE;
+    return FALSE; /* no repeat */
+}
+
+void menu_show(gchar *name, gint x, gint y, gint button, ObClient *client)
 {
     ObMenu *self;
     ObMenuFrame *frame;
-    guint i;
 
     if (!(self = menu_from_name(name))
         || keyboard_interactively_grabbed()) return;
@@ -303,31 +410,41 @@ void menu_show(gchar *name, gint x, gint y, ObClient *client)
 
     menu_frame_hide_all();
 
-    frame = menu_frame_new(self, client);
-    if (client && x < 0 && y < 0) {
-        x = client->frame->area.x + client->frame->size.left;
-        y = client->frame->area.y + client->frame->size.top;
-        menu_frame_move(frame, x, y);
-    } else
-        menu_frame_move(frame,
-                        x - ob_rr_theme->bwidth, y - ob_rr_theme->bwidth);
-    for (i = 0; i < screen_num_monitors; ++i) {
-        Rect *a = screen_physical_area_monitor(i);
-        if (RECT_CONTAINS(*a, x, y)) {
-            frame->monitor = i;
-            break;
+    frame = menu_frame_new(self, 0, client);
+    if (!menu_frame_show_topmenu(frame, x, y, button))
+        menu_frame_free(frame);
+    else if (!button) {
+        /* select the first entry if it's not a submenu and we opened
+         * the menu with the keyboard, and skip all headers */
+        GList *it = frame->entries;
+        while (it) {
+            ObMenuEntryFrame *e = it->data;
+            if (e->entry->type == OB_MENU_ENTRY_TYPE_NORMAL) {
+                menu_frame_select(frame, e, FALSE);
+                break;
+            } else if (e->entry->type == OB_MENU_ENTRY_TYPE_SEPARATOR)
+                it = g_list_next(it);
+            else
+                break;
         }
     }
-    if (!menu_frame_show(frame, NULL))
-        menu_frame_free(frame);
-    else if (frame->entries) {
-        ObMenuEntryFrame *e = frame->entries->data;
-        if (e->entry->type == OB_MENU_ENTRY_TYPE_NORMAL &&
-            e->entry->data.normal.enabled)
-                menu_frame_select(frame, e);
+
+    if (!button)
+        menu_can_hide = TRUE;
+    else {
+        menu_can_hide = FALSE;
+        ob_main_loop_timeout_add(ob_main_loop,
+                                 config_menu_hide_delay * 1000,
+                                 menu_hide_delay_func,
+                                 NULL, g_direct_equal, NULL);
     }
 }
 
+gboolean menu_hide_delay_reached()
+{
+    return menu_can_hide;
+}
+
 static ObMenuEntry* menu_entry_new(ObMenu *menu, ObMenuEntryType type, gint id)
 {
     ObMenuEntry *self;
@@ -335,6 +452,7 @@ static ObMenuEntry* menu_entry_new(ObMenu *menu, ObMenuEntryType type, gint id)
     g_assert(menu);
 
     self = g_new0(ObMenuEntry, 1);
+    self->ref = 1;
     self->type = type;
     self->menu = menu;
     self->id = id;
@@ -351,9 +469,14 @@ static ObMenuEntry* menu_entry_new(ObMenu *menu, ObMenuEntryType type, gint id)
     return self;
 }
 
-void menu_entry_free(ObMenuEntry *self)
+void menu_entry_ref(ObMenuEntry *self)
 {
-    if (self) {
+    ++self->ref;
+}
+
+void menu_entry_unref(ObMenuEntry *self)
+{
+    if (self && --self->ref == 0) {
         switch (self->type) {
         case OB_MENU_ENTRY_TYPE_NORMAL:
             g_free(self->data.normal.label);
@@ -391,31 +514,45 @@ void menu_clear_entries(ObMenu *self)
 #endif
 
     while (self->entries) {
-        menu_entry_free(self->entries->data);
+        menu_entry_unref(self->entries->data);
         self->entries = g_list_delete_link(self->entries, self->entries);
     }
+    self->more_menu->entries = self->entries; /* keep it in sync */
 }
 
 void menu_entry_remove(ObMenuEntry *self)
 {
     self->menu->entries = g_list_remove(self->menu->entries, self);
-    menu_entry_free(self);
+    menu_entry_unref(self);
 }
 
-ObMenuEntry* menu_add_normal(ObMenu *self, gint id, gchar *label,
-                             GSList *actions)
+ObMenuEntry* menu_add_normal(ObMenu *self, gint id, const gchar *label,
+                             GSList *actions, gboolean allow_shortcut)
 {
     ObMenuEntry *e;
 
     e = menu_entry_new(self, OB_MENU_ENTRY_TYPE_NORMAL, id);
-    e->data.normal.label = g_strdup(label);
     e->data.normal.actions = actions;
 
+    menu_entry_set_label(e, label, allow_shortcut);
+
     self->entries = g_list_append(self->entries, e);
+    self->more_menu->entries = self->entries; /* keep it in sync */
     return e;
 }
 
-ObMenuEntry* menu_add_submenu(ObMenu *self, gint id, gchar *submenu)
+ObMenuEntry* menu_get_more(ObMenu *self, guint show_from)
+{
+    ObMenuEntry *e;
+    e = menu_entry_new(self, OB_MENU_ENTRY_TYPE_SUBMENU, -1);
+    /* points to itself */
+    e->data.submenu.name = g_strdup(self->name);
+    e->data.submenu.submenu = self;
+    e->data.submenu.show_from = show_from;
+    return e;
+}
+
+ObMenuEntry* menu_add_submenu(ObMenu *self, gint id, const gchar *submenu)
 {
     ObMenuEntry *e;
 
@@ -423,32 +560,57 @@ ObMenuEntry* menu_add_submenu(ObMenu *self, gint id, gchar *submenu)
     e->data.submenu.name = g_strdup(submenu);
 
     self->entries = g_list_append(self->entries, e);
+    self->more_menu->entries = self->entries; /* keep it in sync */
     return e;
 }
 
-ObMenuEntry* menu_add_separator(ObMenu *self, gint id)
+ObMenuEntry* menu_add_separator(ObMenu *self, gint id, const gchar *label)
 {
     ObMenuEntry *e;
 
     e = menu_entry_new(self, OB_MENU_ENTRY_TYPE_SEPARATOR, id);
 
+    menu_entry_set_label(e, label, FALSE);
+
     self->entries = g_list_append(self->entries, e);
+    self->more_menu->entries = self->entries; /* keep it in sync */
     return e;
 }
 
+void menu_set_show_func(ObMenu *self, ObMenuShowFunc func)
+{
+    self->show_func = func;
+    self->more_menu->show_func = func; /* keep it in sync */
+}
+
+void menu_set_hide_func(ObMenu *self, ObMenuHideFunc func)
+{
+    self->hide_func = func;
+    self->more_menu->hide_func = func; /* keep it in sync */
+}
+
 void menu_set_update_func(ObMenu *self, ObMenuUpdateFunc func)
 {
     self->update_func = func;
+    self->more_menu->update_func = func; /* keep it in sync */
 }
 
 void menu_set_execute_func(ObMenu *self, ObMenuExecuteFunc func)
 {
     self->execute_func = func;
+    self->more_menu->execute_func = func; /* keep it in sync */
 }
 
 void menu_set_destroy_func(ObMenu *self, ObMenuDestroyFunc func)
 {
     self->destroy_func = func;
+    self->more_menu->destroy_func = func; /* keep it in sync */
+}
+
+void menu_set_place_func(ObMenu *self, ObMenuPlaceFunc func)
+{
+    self->place_func = func;
+    self->more_menu->place_func = func; /* keep it in sync */
 }
 
 ObMenuEntry* menu_find_entry_id(ObMenu *self, gint id)
@@ -478,3 +640,27 @@ void menu_find_submenus(ObMenu *self)
             e->data.submenu.submenu = menu_from_name(e->data.submenu.name);
     }
 }
+
+void menu_entry_set_label(ObMenuEntry *self, const gchar *label,
+                          gboolean allow_shortcut)
+{
+    switch (self->type) {
+    case OB_MENU_ENTRY_TYPE_SEPARATOR:
+        g_free(self->data.separator.label);
+        self->data.separator.label = g_strdup(label);
+        break;
+    case OB_MENU_ENTRY_TYPE_NORMAL:
+        g_free(self->data.normal.label);
+        self->data.normal.shortcut =
+            parse_shortcut(label, allow_shortcut, &self->data.normal.label,
+                           &self->data.normal.shortcut_position);
+        break;
+    default:
+        g_assert_not_reached();
+    }
+}
+
+void menu_show_all_shortcuts(ObMenu *self, gboolean show)
+{
+    self->show_all_shortcuts = show;
+}
