@@ -2,7 +2,7 @@
 
    menuframe.h for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -48,28 +48,29 @@ struct _ObMenuFrame
     struct _ObClient *client;
 
     ObMenuFrame *parent;
+    ObMenuEntryFrame *parent_entry;
     ObMenuFrame *child;
 
     GList *entries;
     ObMenuEntryFrame *selected;
 
-    /* If a titlebar is displayed for the menu or not (for top-level menus) */
-    gboolean show_title;
+    /* show entries from the menu starting at this index */
+    guint show_from;
+
+    /* If the submenus are being drawn to the right or the left */
+    gboolean direction_right;
 
     /* On-screen area (including borders!) */
     Rect area;
     Strut item_margin;
     gint inner_w; /* inside the borders */
-    gint title_h; /* includes the bwidth below it */
     gint item_h;  /* height of all normal items */
     gint text_x;  /* offset at which the text appears in the items */
     gint text_w;  /* width of the text area in the items */
+    gint text_h;  /* height of the items */
 
     gint monitor; /* monitor on which to show the menu in xinerama */
 
-    Window title;
-    Window items;
-
     RrAppearance *a_title;
     RrAppearance *a_items;
 };
@@ -79,7 +80,10 @@ struct _ObMenuEntryFrame
     struct _ObMenuEntry *entry;
     ObMenuFrame *frame;
 
+    guint ignore_enters;
+
     Rect area;
+    gint border;
 
     Window window;
     Window icon;
@@ -87,8 +91,9 @@ struct _ObMenuEntryFrame
     Window bullet;
 
     RrAppearance *a_normal;
-    RrAppearance *a_disabled;
     RrAppearance *a_selected;
+    RrAppearance *a_disabled;
+    RrAppearance *a_disabled_selected;
 
     RrAppearance *a_icon;
     RrAppearance *a_mask;
@@ -96,23 +101,38 @@ struct _ObMenuEntryFrame
     RrAppearance *a_bullet_selected;
     RrAppearance *a_separator;
     RrAppearance *a_text_normal;
-    RrAppearance *a_text_disabled;
     RrAppearance *a_text_selected;
+    RrAppearance *a_text_disabled;
+    RrAppearance *a_text_disabled_selected;
+    RrAppearance *a_text_title;
 };
 
-ObMenuFrame* menu_frame_new(struct _ObMenu *menu, struct _ObClient *client);
+extern GHashTable *menu_frame_map;
+
+void menu_frame_startup(gboolean reconfig);
+void menu_frame_shutdown(gboolean reconfig);
+
+ObMenuFrame* menu_frame_new(struct _ObMenu *menu,
+                            guint show_from,
+                            struct _ObClient *client);
 void menu_frame_free(ObMenuFrame *self);
 
 void menu_frame_move(ObMenuFrame *self, gint x, gint y);
-void menu_frame_move_on_screen(ObMenuFrame *self);
+void menu_frame_move_on_screen(ObMenuFrame *self, gint x, gint y,
+                               gint *dx, gint *dy);
 
-gboolean menu_frame_show(ObMenuFrame *self, ObMenuFrame *parent);
-void menu_frame_hide(ObMenuFrame *self);
+gboolean menu_frame_show_topmenu(ObMenuFrame *self, gint x, gint y,
+                                 gint button);
+gboolean menu_frame_show_submenu(ObMenuFrame *self, ObMenuFrame *parent,
+                                 ObMenuEntryFrame *parent_entry);
 
 void menu_frame_hide_all();
 void menu_frame_hide_all_client(struct _ObClient *client);
 
-void menu_frame_select(ObMenuFrame *self, ObMenuEntryFrame *entry);
+void menu_frame_render(ObMenuFrame *self);
+
+void menu_frame_select(ObMenuFrame *self, ObMenuEntryFrame *entry,
+                       gboolean immediate);
 void menu_frame_select_previous(ObMenuFrame *self);
 void menu_frame_select_next(ObMenuFrame *self);
 
@@ -121,6 +141,6 @@ ObMenuEntryFrame* menu_entry_frame_under(gint x, gint y);
 
 void menu_entry_frame_show_submenu(ObMenuEntryFrame *self);
 
-void menu_entry_frame_execute(ObMenuEntryFrame *self, guint state);
+void menu_entry_frame_execute(ObMenuEntryFrame *self, guint state, Time time);
 
 #endif
