@@ -2,7 +2,7 @@
 
    place.c for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,16 +23,61 @@
 #include "frame.h"
 #include "focus.h"
 #include "config.h"
+#include "debug.h"
 
-static Rect* pick_head(ObClient *c)
+static void add_choice(guint *choice, guint mychoice)
 {
+    guint i;
+    for (i = 0; i < screen_num_monitors; ++i) {
+        if (choice[i] == mychoice)
+            return;
+        else if (choice[i] == screen_num_monitors) {
+            choice[i] = mychoice;
+            return;
+        }
+    }
+}
+
+static Rect *pick_pointer_head(ObClient *c)
+{
+    guint i;
+    gint px, py;
+
+    screen_pointer_pos(&px, &py);
+     
+    for (i = 0; i < screen_num_monitors; ++i) {  
+        if (RECT_CONTAINS(*screen_physical_area_monitor(i), px, py)) {
+            return screen_area_monitor(c->desktop, i);
+        }
+    }
+    g_assert_not_reached();
+}
+
+/*! Pick a monitor to place a window on.
+  The returned array value should be freed with g_free. The areas within the
+  array should not be freed. */
+static Rect **pick_head(ObClient *c)
+{
+    Rect **area;
+    guint *choice;
+    guint i;
+    gint px, py;
+
+    area = g_new(Rect*, screen_num_monitors);
+    choice = g_new(guint, screen_num_monitors);
+    for (i = 0; i < screen_num_monitors; ++i)
+        choice[i] = screen_num_monitors; /* make them all invalid to start */
+
     /* try direct parent first */
-    if (c->transient_for && c->transient_for != OB_TRAN_GROUP) {
-        return screen_area_monitor(c->desktop,
-                                   client_monitor(c->transient_for));
+    if (c->transient_for && c->transient_for != OB_TRAN_GROUP &&
+        client_normal(c->transient_for))
+    {
+        add_choice(choice, client_monitor(c->transient_for));
+        ob_debug("placement adding choice %d for parent\n",
+                 client_monitor(c->transient_for));
     }
 
-    /* more than one guy in his group (more than just him) */
+    /* more than one window in its group (more than just this window) */
     if (client_has_group_siblings(c)) {
         GSList *it;
 
@@ -42,41 +87,69 @@ static Rect* pick_head(ObClient *c)
             if (itc != c &&
                 (itc->desktop == c->desktop ||
                  itc->desktop == DESKTOP_ALL || c->desktop == DESKTOP_ALL))
-                return screen_area_monitor(c->desktop,
-                                           client_monitor(it->data));
+            {
+                add_choice(choice, client_monitor(it->data));
+                ob_debug("placement adding choice %d for group sibling\n",
+                         client_monitor(it->data));
+            }
         }
 
         /* try on all desktops */
         for (it = c->group->members; it; it = g_slist_next(it)) {
             ObClient *itc = it->data;            
-            if (itc != c)
-                return screen_area_monitor(c->desktop,
-                                           client_monitor(it->data));
+            if (itc != c) {
+                add_choice(choice, client_monitor(it->data));
+                ob_debug("placement adding choice %d for group sibling on "
+                         "another desktop\n", client_monitor(it->data));
+            }
         }
     }
 
-    return NULL;
+    if (focus_client) {
+        add_choice(choice, client_monitor(focus_client));
+        ob_debug("placement adding choice %d for focused window\n",
+                 client_monitor(focus_client));
+    }
+
+    screen_pointer_pos(&px, &py);
+
+    for (i = 0; i < screen_num_monitors; i++)
+        if (RECT_CONTAINS(*screen_physical_area_monitor(i), px, py)) {
+            add_choice(choice, i);
+            ob_debug("placement adding choice %d for mouse pointer\n", i);
+            break;
+        }
+
+    /* add any leftover choices */
+    for (i = 0; i < screen_num_monitors; ++i)
+        add_choice(choice, i);
+
+    for (i = 0; i < screen_num_monitors; ++i)
+        area[i] = screen_area_monitor(c->desktop, choice[i]);
+
+    return area;
 }
 
 static gboolean place_random(ObClient *client, gint *x, gint *y)
 {
     gint l, r, t, b;
-    Rect *area;
+    Rect **areas;
+    guint i;
 
-    area = pick_head(client);
-    if (!area)
-        area = screen_area_monitor(client->desktop,
-                                   g_random_int_range(0, screen_num_monitors));
+    areas = pick_head(client);
+    i = g_random_int_range(0, screen_num_monitors);
 
-    l = area->x;
-    t = area->y;
-    r = area->x + area->width - client->frame->area.width;
-    b = area->y + area->height - client->frame->area.height;
+    l = areas[i]->x;
+    t = areas[i]->y;
+    r = areas[i]->x + areas[i]->width - client->frame->area.width;
+    b = areas[i]->y + areas[i]->height - client->frame->area.height;
 
     if (r > l) *x = g_random_int_range(l, r + 1);
-    else       *x = 0;
+    else       *x = areas[i]->x;
     if (b > t) *y = g_random_int_range(t, b + 1);
-    else       *y = 0;
+    else       *y = areas[i]->y;
+
+    g_free(areas);
 
     return TRUE;
 }
@@ -197,56 +270,67 @@ typedef enum
 } ObSmartType;
 
 #define SMART_IGNORE(placer, c) \
-    (placer == c || !c->frame->visible || c->shaded || !client_normal(c) || \
+    (placer == c || c->shaded || !c->frame->visible || \
+     c->type == OB_CLIENT_TYPE_SPLASH || c->type == OB_CLIENT_TYPE_DESKTOP || \
+     ((c->type == OB_CLIENT_TYPE_MENU || c->type == OB_CLIENT_TYPE_TOOLBAR) &&\
+      client_has_parent(c)) || \
      (c->desktop != DESKTOP_ALL && \
       c->desktop != (placer->desktop == DESKTOP_ALL ? \
                      screen_desktop : placer->desktop)))
 
 static gboolean place_smart(ObClient *client, gint *x, gint *y,
-                            ObSmartType type)
+                            ObSmartType type, gboolean ignore_max)
 {
-    guint i;
     gboolean ret = FALSE;
     GSList *spaces = NULL, *sit;
     GList *it;
+    Rect **areas;
+    guint i;
 
-    for (i = 0; i < screen_num_monitors; ++i)
-        spaces = area_add(spaces, screen_area_monitor(client->desktop, i));
-
-    /* stay out from under windows in higher layers */
-    for (it = stacking_list; it; it = g_list_next(it)) {
-        ObClient *c;
-
-        if (WINDOW_IS_CLIENT(it->data)) {
-            c = it->data;
-            if (c->fullscreen)
-                continue;
-        } else
-            continue;
-
-        if (c->layer > client->layer) {
-            if (!SMART_IGNORE(client, c))
-                spaces = area_remove(spaces, &c->frame->area);
-        } else
-            break;
+    if (type == SMART_GROUP) {
+        /* has to be more than me in the group */
+        if (!client_has_group_siblings(client))
+            return FALSE;
     }
 
-    if (client->type == OB_CLIENT_TYPE_NORMAL) {
+    areas = pick_head(client);
+
+    for (i = 0; i < screen_num_monitors && !ret; ++i) {
+        spaces = area_add(spaces, areas[i]);
+
+        /* stay out from under windows in higher layers */
+        for (it = stacking_list; it; it = g_list_next(it)) {
+            ObClient *c;
+
+            if (WINDOW_IS_CLIENT(it->data)) {
+                c = it->data;
+                if (ignore_max &&
+                    (c->fullscreen || (c->max_vert && c->max_horz)))
+                    continue;
+            } else
+                continue;
+
+            if (c->layer > client->layer) {
+                if (!SMART_IGNORE(client, c))
+                    spaces = area_remove(spaces, &c->frame->area);
+            } else
+                break;
+        }
+
         if (type == SMART_FULL || type == SMART_FOCUSED) {
             gboolean found_foc = FALSE, stop = FALSE;
             ObClient *foc;
-            GList *list;
 
-            list = focus_order[client->desktop == DESKTOP_ALL ?
-                               screen_desktop : client->desktop];
-            foc = list ? list->data : NULL;
+            foc = focus_order_find_first(client->desktop == DESKTOP_ALL ?
+                                         screen_desktop : client->desktop);
 
             for (; it && !stop; it = g_list_next(it)) {
                 ObClient *c;
 
                 if (WINDOW_IS_CLIENT(it->data)) {
                     c = it->data;
-                    if (c->fullscreen)
+                    if (ignore_max &&
+                        (c->fullscreen || (c->max_vert && c->max_horz)))
                         continue;
                 } else
                     continue;
@@ -263,10 +347,6 @@ static gboolean place_smart(ObClient *client, gint *x, gint *y,
                     found_foc = TRUE;
             }
         } else if (type == SMART_GROUP) {
-            /* has to be more than me in the group */
-            if (!client_has_group_siblings(client))
-                return FALSE;
-
             for (sit = client->group->members; sit; sit = g_slist_next(sit)) {
                 ObClient *c = sit->data;
                 if (!SMART_IGNORE(client, c))
@@ -274,53 +354,48 @@ static gboolean place_smart(ObClient *client, gint *x, gint *y,
             }
         } else
             g_assert_not_reached();
-    }
 
-    spaces = g_slist_sort_with_data(spaces, area_cmp, client);
+        spaces = g_slist_sort_with_data(spaces, area_cmp, client);
 
-    for (sit = spaces; sit; sit = g_slist_next(sit)) {
-        Rect *r = sit->data;
+        for (sit = spaces; sit; sit = g_slist_next(sit)) {
+            Rect *r = sit->data;
 
-        if (!ret) {
-            if (r->width >= client->frame->area.width &&
-                r->height >= client->frame->area.height) {
-                ret = TRUE;
-                if (client->type == OB_CLIENT_TYPE_DIALOG ||
-                    type != SMART_FULL)
-                {
-                    *x = r->x + (r->width - client->frame->area.width) / 2;
-                    *y = r->y + (r->height - client->frame->area.height) / 2;
-                } else {
-                    *x = r->x;
-                    *y = r->y;
+            if (!ret) {
+                if (r->width >= client->frame->area.width &&
+                    r->height >= client->frame->area.height) {
+                    ret = TRUE;
+                    if (client->type == OB_CLIENT_TYPE_DIALOG ||
+                        type != SMART_FULL)
+                    {
+                        *x = r->x + (r->width - client->frame->area.width)/2;
+                        *y = r->y + (r->height - client->frame->area.height)/2;
+                    } else {
+                        *x = r->x;
+                        *y = r->y;
+                    }
                 }
             }
-        }
 
-        g_free(r);
+            g_free(r);
+        }
+        g_slist_free(spaces);
+        spaces = NULL;
     }
-    g_slist_free(spaces);
+
+    g_free(areas);
 
     return ret;
 }
 
 static gboolean place_under_mouse(ObClient *client, gint *x, gint *y)
 {
-    guint i;
     gint l, r, t, b;
     gint px, py;
     Rect *area;
 
+    area = pick_pointer_head(client);
     screen_pointer_pos(&px, &py);
 
-    for (i = 0; i < screen_num_monitors; ++i) {
-        area = screen_area_monitor(client->desktop, i);
-        if (RECT_CONTAINS(*area, px, py))
-            break;
-    }
-    if (i == screen_num_monitors)
-        area = screen_area_monitor(client->desktop, 0);
-
     l = area->x;
     t = area->y;
     r = area->x + area->width - client->frame->area.width;
@@ -334,29 +409,27 @@ static gboolean place_under_mouse(ObClient *client, gint *x, gint *y)
     return TRUE;
 }
 
-static gboolean place_per_app_setting(ObClient *client, gint *x, gint *y, ObAppSettings *settings)
+static gboolean place_per_app_setting(ObClient *client, gint *x, gint *y,
+                                      ObAppSettings *settings)
 {
-    gint px, py, i;
-    Rect *screen;
+    Rect *screen = NULL;
 
     if (!settings || (settings && !settings->pos_given))
         return FALSE;
 
     /* Find which head the pointer is on */
-    if (settings->head == -1 && screen_num_monitors > 1) {
-        screen_pointer_pos(&px, &py);
-
-        for (i = 0; i < screen_num_monitors; i++) {
-            screen = screen_area_monitor(client->desktop, i);
-            if (RECT_CONTAINS(*screen, px, py))
-                break;
-        }
-
-        if (i == screen_num_monitors)
-            screen = screen_area_monitor(client->desktop, 0);
+    if (settings->monitor == 0)
+        screen = pick_pointer_head(client);
+    else if (settings->monitor > 0 &&
+             (guint)settings->monitor <= screen_num_monitors)
+        screen = screen_area_monitor(client->desktop,
+                                     (guint)settings->monitor - 1);
+    else {
+        Rect **all = NULL;
+        all = pick_head(client);
+        screen = all[0];
+        g_free(all); /* the areas themselves don't need to be freed */
     }
-    else
-        screen = screen_area_monitor(client->desktop, settings->head);
 
     if (settings->center_x)
         *x = screen->x + screen->width / 2 - client->area.width / 2;
@@ -371,24 +444,32 @@ static gboolean place_per_app_setting(ObClient *client, gint *x, gint *y, ObAppS
     return TRUE;
 }
 
-static gboolean place_transient(ObClient *client, gint *x, gint *y)
+static gboolean place_transient_splash(ObClient *client, gint *x, gint *y)
 {
-    if (client->transient_for) {
-        if (client->transient_for != OB_TRAN_GROUP) {
+    if (client->transient_for && client->type == OB_CLIENT_TYPE_DIALOG) {
+        if (client->transient_for != OB_TRAN_GROUP &&
+            client_normal(client->transient_for) &&
+            !client->iconic)
+        {
             ObClient *c = client;
             ObClient *p = client->transient_for;
-            *x = (p->frame->area.width - c->frame->area.width) / 2 +
-                p->frame->area.x;
-            *y = (p->frame->area.height - c->frame->area.height) / 2 +
-                p->frame->area.y;
-            return TRUE;
+
+            if (client_normal(p)) {
+                *x = (p->frame->area.width - c->frame->area.width) / 2 +
+                    p->frame->area.x;
+                *y = (p->frame->area.height - c->frame->area.height) / 2 +
+                    p->frame->area.y;
+                return TRUE;
+            }
         } else {
             GSList *it;
             gboolean first = TRUE;
             gint l, r, t, b;
             for (it = client->group->members; it; it = g_slist_next(it)) {
                 ObClient *m = it->data;
-                if (!(m == client || m->transient_for)) {
+                if (!(m == client || m->transient_for) && client_normal(m) &&
+                    !m->iconic)
+                {
                     if (first) {
                         l = RECT_LEFT(m->frame->area);
                         t = RECT_TOP(m->frame->area);
@@ -410,27 +491,46 @@ static gboolean place_transient(ObClient *client, gint *x, gint *y)
             }
         }
     }
+
+    if ((client->transient && client->type == OB_CLIENT_TYPE_DIALOG)
+        || client->type == OB_CLIENT_TYPE_SPLASH)
+    {
+        Rect **areas;
+
+        areas = pick_head(client);
+
+        *x = (areas[0]->width - client->frame->area.width) / 2 + areas[0]->x;
+        *y = (areas[0]->height - client->frame->area.height) / 2 + areas[0]->y;
+
+        g_free(areas);
+        return TRUE;
+    }
+
     return FALSE;
 }
 
 /* Return TRUE if we want client.c to enforce on-screen-keeping */
-gboolean place_client(ObClient *client, gint *x, gint *y, ObAppSettings *settings)
+gboolean place_client(ObClient *client, gint *x, gint *y,
+                      ObAppSettings *settings)
 {
     gboolean ret = FALSE;
     if (client->positioned)
         return FALSE;
-    if (place_transient(client, x, y))
+    if (place_transient_splash(client, x, y))
         ret = TRUE;
     else if (!(
         place_per_app_setting(client, x, y, settings) ||
         ((config_place_policy == OB_PLACE_POLICY_MOUSE) ?
          place_under_mouse(client, x, y) :
-         place_smart(client, x, y, SMART_FULL)    ||
-         place_smart(client, x, y, SMART_GROUP)   ||
-         place_smart(client, x, y, SMART_FOCUSED) ||
+         place_smart(client, x, y, SMART_FULL, FALSE)   ||
+         place_smart(client, x, y, SMART_FULL, TRUE)    ||
+         place_smart(client, x, y, SMART_GROUP, FALSE)  ||
+         place_smart(client, x, y, SMART_GROUP, TRUE)   ||
+         place_smart(client, x, y, SMART_FOCUSED, TRUE) ||
          place_random(client, x, y))))
         g_assert_not_reached(); /* the last one better succeed */
     /* get where the client should be */
-    frame_frame_gravity(client->frame, x, y);
+    frame_frame_gravity(client->frame, x, y,
+                        client->area.width, client->area.height);
     return ret;
 }
