@@ -2,7 +2,7 @@
 
    prop.c for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -48,10 +48,15 @@ void prop_startup()
     CREATE(wm_icon_name, "WM_ICON_NAME");
     CREATE(wm_class, "WM_CLASS");
     CREATE(wm_window_role, "WM_WINDOW_ROLE");
+    CREATE(wm_client_machine, "WM_CLIENT_MACHINE");
+    CREATE(wm_command, "WM_COMMAND");
+    CREATE(wm_client_leader, "WM_CLIENT_LEADER");
     CREATE(motif_wm_hints, "_MOTIF_WM_HINTS");
 
     CREATE(sm_client_id, "SM_CLIENT_ID");
 
+    CREATE(net_wm_full_placement, "_NET_WM_FULL_PLACEMENT");
+
     CREATE(net_supported, "_NET_SUPPORTED");
     CREATE(net_client_list, "_NET_CLIENT_LIST");
     CREATE(net_client_list_stacking, "_NET_CLIENT_LIST_STACKING");
@@ -61,6 +66,7 @@ void prop_startup()
     CREATE(net_current_desktop, "_NET_CURRENT_DESKTOP");
     CREATE(net_desktop_names, "_NET_DESKTOP_NAMES");
     CREATE(net_active_window, "_NET_ACTIVE_WINDOW");
+/*    CREATE(net_restack_window, "_NET_RESTACK_WINDOW");*/
     CREATE(net_workarea, "_NET_WORKAREA");
     CREATE(net_supporting_wm_check, "_NET_SUPPORTING_WM_CHECK");
     CREATE(net_desktop_layout, "_NET_DESKTOP_LAYOUT");
@@ -69,6 +75,8 @@ void prop_startup()
     CREATE(net_close_window, "_NET_CLOSE_WINDOW");
     CREATE(net_wm_moveresize, "_NET_WM_MOVERESIZE");
     CREATE(net_moveresize_window, "_NET_MOVERESIZE_WINDOW");
+    CREATE(net_request_frame_extents, "_NET_REQUEST_FRAME_EXTENTS");
+    CREATE(net_restack_window, "_NET_RESTACK_WINDOW");
 
     CREATE(net_startup_id, "_NET_STARTUP_ID");
 
@@ -82,11 +90,20 @@ void prop_startup()
     CREATE(net_wm_strut, "_NET_WM_STRUT");
     CREATE(net_wm_strut_partial, "_NET_WM_STRUT_PARTIAL");
     CREATE(net_wm_icon, "_NET_WM_ICON");
+    CREATE(net_wm_icon_geometry, "_NET_WM_ICON_GEOMETRY");
 /*   CREATE(net_wm_pid, "_NET_WM_PID"); */
     CREATE(net_wm_allowed_actions, "_NET_WM_ALLOWED_ACTIONS");
+    CREATE(net_wm_user_time, "_NET_WM_USER_TIME");
+    CREATE(net_wm_user_time_window, "_NET_WM_USER_TIME_WINDOW");
+    CREATE(kde_net_wm_frame_strut, "_KDE_NET_WM_FRAME_STRUT");
+    CREATE(net_frame_extents, "_NET_FRAME_EXTENTS");
 
 /*   CREATE(net_wm_ping, "_NET_WM_PING"); */
-  
+#ifdef SYNC
+    CREATE(net_wm_sync_request, "_NET_WM_SYNC_REQUEST");
+    CREATE(net_wm_sync_request_counter, "_NET_WM_SYNC_REQUEST_COUNTER");
+#endif
+
     CREATE(net_wm_window_type_desktop, "_NET_WM_WINDOW_TYPE_DESKTOP");
     CREATE(net_wm_window_type_dock, "_NET_WM_WINDOW_TYPE_DOCK");
     CREATE(net_wm_window_type_toolbar, "_NET_WM_WINDOW_TYPE_TOOLBAR");
@@ -107,19 +124,22 @@ void prop_startup()
     prop_atoms.net_wm_moveresize_move = 8;
     prop_atoms.net_wm_moveresize_size_keyboard = 9;
     prop_atoms.net_wm_moveresize_move_keyboard = 10;
+    prop_atoms.net_wm_moveresize_cancel = 11;
 
     CREATE(net_wm_action_move, "_NET_WM_ACTION_MOVE");
     CREATE(net_wm_action_resize, "_NET_WM_ACTION_RESIZE");
     CREATE(net_wm_action_minimize, "_NET_WM_ACTION_MINIMIZE");
     CREATE(net_wm_action_shade, "_NET_WM_ACTION_SHADE");
-    CREATE(net_wm_action_stick, "_NET_WM_ACTION_STICK");
     CREATE(net_wm_action_maximize_horz, "_NET_WM_ACTION_MAXIMIZE_HORZ");
     CREATE(net_wm_action_maximize_vert, "_NET_WM_ACTION_MAXIMIZE_VERT");
     CREATE(net_wm_action_fullscreen, "_NET_WM_ACTION_FULLSCREEN");
     CREATE(net_wm_action_change_desktop, "_NET_WM_ACTION_CHANGE_DESKTOP");
     CREATE(net_wm_action_close, "_NET_WM_ACTION_CLOSE");
+    CREATE(net_wm_action_above, "_NET_WM_ACTION_ABOVE");
+    CREATE(net_wm_action_below, "_NET_WM_ACTION_BELOW");
+
     CREATE(net_wm_state_modal, "_NET_WM_STATE_MODAL");
-    CREATE(net_wm_state_sticky, "_NET_WM_STATE_STICKY");
+/*    CREATE(net_wm_state_sticky, "_NET_WM_STATE_STICKY");*/
     CREATE(net_wm_state_maximized_vert, "_NET_WM_STATE_MAXIMIZED_VERT");
     CREATE(net_wm_state_maximized_horz, "_NET_WM_STATE_MAXIMIZED_HORZ");
     CREATE(net_wm_state_shaded, "_NET_WM_STATE_SHADED");
@@ -129,6 +149,7 @@ void prop_startup()
     CREATE(net_wm_state_fullscreen, "_NET_WM_STATE_FULLSCREEN");
     CREATE(net_wm_state_above, "_NET_WM_STATE_ABOVE");
     CREATE(net_wm_state_below, "_NET_WM_STATE_BELOW");
+    CREATE(net_wm_state_demands_attention, "_NET_WM_STATE_DEMANDS_ATTENTION");
   
     prop_atoms.net_wm_state_add = 1;
     prop_atoms.net_wm_state_remove = 0;
@@ -141,17 +162,19 @@ void prop_startup()
     prop_atoms.net_wm_bottomright = 2;
     prop_atoms.net_wm_bottomleft = 3;
 
-    CREATE(kde_net_wm_frame_strut, "_KDE_NET_WM_FRAME_STRUT");
     CREATE(kde_wm_change_state, "_KDE_WM_CHANGE_STATE");
     CREATE(kde_net_wm_window_type_override,"_KDE_NET_WM_WINDOW_TYPE_OVERRIDE");
 
-    CREATE(kwm_win_icon, "KWM_WIN_ICON");
-  
+/*
     CREATE(rootpmapid, "_XROOTPMAP_ID");
     CREATE(esetrootid, "ESETROOT_PMAP_ID");
+*/
 
     CREATE(openbox_pid, "_OPENBOX_PID");
+    CREATE(ob_theme, "_OB_THEME");
+    CREATE(ob_wm_action_undecorate, "_OB_WM_ACTION_UNDECORATE");
     CREATE(ob_wm_state_undecorated, "_OB_WM_STATE_UNDECORATED");
+    CREATE(ob_control, "_OB_CONTROL");
 }
 
 #include <X11/Xutil.h>
@@ -293,8 +316,7 @@ gboolean prop_get_string_locale(Window win, Atom prop, gchar **ret)
     gchar *s;
 
     if (get_stringlist(win, prop, &list, &nstr) && nstr) {
-        s = g_convert(list[0], strlen(list[0]), "UTF-8", "ISO-8859-1",
-                      NULL, NULL, NULL);
+        s = g_locale_to_utf8(list[0], -1, NULL, NULL, NULL);
         XFreeStringList(list);
         if (s) {
             *ret = s;
@@ -313,7 +335,7 @@ gboolean prop_get_strings_locale(Window win, Atom prop, gchar ***ret)
     if (get_all(win, prop, prop_atoms.string, 8, (guchar**)&raw, &num)) {
 
         p = raw;
-        while (p < raw + num - 1) {
+        while (p < raw + num) {
             ++count;
             strs = g_slist_append(strs, p);
             p += strlen(p) + 1; /* next string */
@@ -323,8 +345,7 @@ gboolean prop_get_strings_locale(Window win, Atom prop, gchar ***ret)
         (*ret)[count] = NULL; /* null terminated list */
 
         for (i = 0, it = strs; it; ++i, it = g_slist_next(it)) {
-            (*ret)[i] = g_convert(it->data, -1, "UTF-8", "ISO-8859-1",
-                                  NULL, NULL, NULL);
+            (*ret)[i] = g_locale_to_utf8(it->data, -1, NULL, NULL, NULL);
             /* make sure translation did not fail */
             if (!(*ret)[i])
                 (*ret)[i] = g_strdup("");
@@ -363,7 +384,7 @@ gboolean prop_get_strings_utf8(Window win, Atom prop, gchar ***ret)
     if (get_all(win, prop, prop_atoms.utf8, 8, (guchar**)&raw, &num)) {
 
         p = raw;
-        while (p < raw + num - 1) {
+        while (p < raw + num) {
             ++count;
             strs = g_slist_append(strs, p);
             p += strlen(p) + 1; /* next string */
@@ -397,10 +418,10 @@ void prop_set_array32(Window win, Atom prop, Atom type, gulong *val,
                     (guchar*)val, num);
 }
 
-void prop_set_string_utf8(Window win, Atom prop, gchar *val)
+void prop_set_string_utf8(Window win, Atom prop, const gchar *val)
 {
     XChangeProperty(ob_display, win, prop, prop_atoms.utf8, 8,
-                    PropModeReplace, (guchar*)val, strlen(val));
+                    PropModeReplace, (const guchar*)val, strlen(val));
 }
 
 void prop_set_strings_utf8(Window win, Atom prop, gchar **strs)
@@ -436,6 +457,7 @@ void prop_message(Window about, Atom messagetype, glong data0, glong data1,
     ce.xclient.data.l[1] = data1;
     ce.xclient.data.l[2] = data2;
     ce.xclient.data.l[3] = data3;
+    ce.xclient.data.l[4] = 0;
     XSendEvent(ob_display, RootWindow(ob_display, ob_screen), FALSE,
                mask, &ce);
 }
