@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: nil; tab-width: 4; c-basic-offset: 4; -*-
 
    screen.h for the Openbox window manager
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -38,14 +38,16 @@ extern guint screen_last_desktop;
 extern gboolean screen_showing_desktop;
 /*! The support window also used for focus and stacking */
 extern Window screen_support_win;
+/*! The last time at which the user changed desktops */
+extern Time screen_desktop_user_time;
 
-typedef struct DesktopLayout {
+typedef struct ObDesktopLayout {
     ObOrientation orientation;
     ObCorner start_corner;
     guint rows;
     guint columns;
-} DesktopLayout;
-extern DesktopLayout screen_desktop_layout;
+} ObDesktopLayout;
+extern ObDesktopLayout screen_desktop_layout;
 
 /*! An array of gchar*'s which are desktop names in UTF-8 format */
 extern gchar **screen_desktop_names;
@@ -64,7 +66,7 @@ void screen_resize();
 /*! Change the number of available desktops */
 void screen_set_num_desktops(guint num);
 /*! Change the current desktop */
-void screen_set_desktop(guint num);
+void screen_set_desktop(guint num, gboolean dofocus);
 /*! Interactively change desktops */
 guint screen_cycle_desktop(ObDirection dir, gboolean wrap, gboolean linear,
                            gboolean dialog, gboolean done, gboolean cancel);
@@ -73,8 +75,12 @@ guint screen_cycle_desktop(ObDirection dir, gboolean wrap, gboolean linear,
 void screen_desktop_popup(guint d, gboolean show);
 
 /*! Shows and focuses the desktop and hides all the client windows, or
-  returns to the normal state, showing client windows. */
-void screen_show_desktop(gboolean show);
+  returns to the normal state, showing client windows.
+  @param If show_only is non-NULL, then only that client is shown (assuming
+         show is FALSE (restoring from show-desktop mode), and the rest are
+         iconified.
+*/
+void screen_show_desktop(gboolean show, struct _ObClient *show_only);
 
 /*! Updates the desktop layout from the root property if available */
 void screen_update_layout();
@@ -96,10 +102,17 @@ Rect *screen_area(guint desktop);
 
 Rect *screen_area_monitor(guint desktop, guint head);
 
+/*! Determines which physical monitor a rectangle is on by calculating the
+    area of the part of the rectable on each monitor.  The number of the
+    monitor containing the greatest area of the rectangle is returned.*/
+guint screen_find_monitor(Rect *search);
+
 /*! Sets the root cursor. This function decides which cursor to use, but you
   gotta call it to let it know it should change. */
 void screen_set_root_cursor();
 
+/*! Gives back the pointer's position in x and y. Returns TRUE if the pointer
+  is on this screen and FALSE if it is on another screen. */
 gboolean screen_pointer_pos(gint *x, gint *y);
 
 #endif
