@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: nil; tab-width: 4; c-basic-offset: 4; -*-
 
    startupnotify.h for the Openbox window manager
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #define ob__startupnotify_h
 
 #include <glib.h>
+#include <X11/Xlib.h>
 
 void sn_startup(gboolean reconfig);
 void sn_shutdown(gboolean reconfig);
@@ -27,10 +28,20 @@ void sn_shutdown(gboolean reconfig);
 gboolean sn_app_starting();
 
 /*! Notify that an app has started */
-void sn_app_started(gchar *wmclass);
+Time sn_app_started(const gchar *id, const gchar *wmclass);
 
 /*! Get the desktop requested via the startup-notiication protocol if one
   was requested */
 gboolean sn_get_desktop(gchar *id, guint *desktop);
 
+/* Get the environment to run the program in, with startup notification */
+void sn_setup_spawn_environment(gchar *program, gchar *name,
+                                gchar *icon_name, gint desktop,
+                                Time time);
+
+/* Tell startup notification we're not actually running the program we
+   told it we were
+*/
+void sn_spawn_cancel();
+
 #endif
