@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: nil; tab-width: 4; c-basic-offset: 4; -*-
 
    font.h for the Openbox window manager
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
    Copyright (c) 2003        Derek Foreman
 
    This program is free software; you can redistribute it and/or modify
@@ -21,26 +21,22 @@
 #define __font_h
 #include "render.h"
 #include "geom.h"
-#ifdef USE_PANGO
 #include <pango/pango.h>
-#include <pango/pangoxft.h>
-#endif /* USE_PANGO */
 
 struct _RrFont {
     const RrInstance *inst;
-    XftFont *xftfont;
-    gint elipses_length;
-    gint shadow;
-    gchar tint;
-    gint offset;
-#ifdef USE_PANGO
-    PangoFontDescription *pango_font_description;
-    gint pango_ascent;
-    gint pango_descent;
-#endif /* USE_PANGO */
+    gint ref;
+    PangoFontDescription *font_desc; 
+    PangoLayout *layout; /*!< Used for measuring and rendering strings */
+    PangoAttribute *shortcut_underline; /*< For underlining the shortcut key */
+    gint ascent; /*!< The font's ascent in pango-units */
+    gint descent; /*!< The font's descent in pango-units */
 };
 
-RrFont *RrFontOpen(const RrInstance *inst, gchar *fontstring);
-void RrFontClose(RrFont *f);
 void RrFontDraw(XftDraw *d, RrTextureText *t, RrRect *position);
+
+/*! Increment the references for this font, RrFontClose will decrement until 0
+  and then really close it */
+void RrFontRef(RrFont *f);
+
 #endif /* __font_h */
