@@ -2,7 +2,7 @@
 
    gradient.c for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
    Copyright (c) 2003        Derek Foreman
 
    This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,7 @@
 #include <glib.h>
 
 static void highlight(RrPixel32 *x, RrPixel32 *y, gboolean raised);
+static void gradient_parentrelative(RrAppearance *a, gint w, gint h);
 static void gradient_solid(RrAppearance *l, gint w, gint h);
 static void gradient_splitvertical(RrAppearance *a, gint w, gint h);
 static void gradient_vertical(RrSurface *sf, gint w, gint h);
@@ -41,6 +42,9 @@ void RrRender(RrAppearance *a, gint w, gint h)
     gint off, x;
 
     switch (a->surface.grad) {
+    case RR_SURFACE_PARENTREL:
+        gradient_parentrelative(a, w, h);
+        break;
     case RR_SURFACE_SOLID:
         gradient_solid(a, w, h);
         break;
@@ -190,6 +194,57 @@ static void create_bevel_colors(RrAppearance *l)
     l->surface.bevel_dark = RrColorNew(l->inst, r, g, b);
 }
 
+static void gradient_parentrelative(RrAppearance *a, gint w, gint h)
+{
+    RrPixel32 *source, *dest;
+    gint sw, sh, partial_w, partial_h, i;
+
+    g_assert (a->surface.parent);
+    g_assert (a->surface.parent->w);
+
+    sw = a->surface.parent->w;
+    sh = a->surface.parent->h;
+
+    /* This is a little hack. When a texture is parentrelative, and the same
+       area as the parent, and has a bevel, it will draw its bevel on top
+       of the parent's, amplifying it. So instead, rerender the child with
+       the parent's settings, but the child's bevel and interlace */
+    if (a->surface.relief != RR_RELIEF_FLAT &&
+        (a->surface.parent->surface.relief != RR_RELIEF_FLAT ||
+         a->surface.parent->surface.border) &&
+        !a->surface.parentx && !a->surface.parenty &&
+        sw == w && sh == h)
+    {
+        RrSurface old = a->surface;
+        a->surface = a->surface.parent->surface;
+
+        /* turn these off for the parent */
+        a->surface.relief = RR_RELIEF_FLAT;
+        a->surface.border = FALSE;
+
+        a->surface.pixel_data = old.pixel_data;
+
+        RrRender(a, w, h);
+        a->surface = old;
+    } else {
+        source = (a->surface.parent->surface.pixel_data +
+                  a->surface.parentx + sw * a->surface.parenty);
+        dest = a->surface.pixel_data;
+
+        if (a->surface.parentx + w > sw) {
+            partial_w = sw - a->surface.parentx;
+        } else partial_w = w;
+
+        if (a->surface.parenty + h > sh) {
+            partial_h = sh - a->surface.parenty;
+        } else partial_h = h;
+
+        for (i = 0; i < partial_h; i++, source += sw, dest += w) {
+            memcpy(dest, source, partial_w * sizeof(RrPixel32));
+        }
+    }
+}
+
 static void gradient_solid(RrAppearance *l, gint w, gint h) 
 {
     gint i;
@@ -371,6 +426,9 @@ static void gradient_splitvertical(RrAppearance *a, gint w, gint h)
     RrPixel32 current;
     RrColor *primary_light, *secondary_light;
 
+    VARS(y1);
+    VARS(y3);
+
     r = sf->primary->r;
     r += r >> 2;
     g = sf->primary->g;
@@ -393,10 +451,7 @@ static void gradient_splitvertical(RrAppearance *a, gint w, gint h)
     if (b > 0xFF) b = 0xFF;
     secondary_light = RrColorNew(a->inst, r, g, b);
 
-    VARS(y1);
     SETUP(y1, primary_light, sf->primary, (h / 2) -1);
-  
-    VARS(y3);
     SETUP(y3, sf->secondary, secondary_light,  (h / 2) -1);
 
     for (y1 = h - 1; y1 > (h / 2) -1; --y1) {  /* 0 -> h-1 */
