@@ -2,7 +2,7 @@
 
    render.c for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
    Copyright (c) 2003        Derek Foreman
 
    This program is free software; you can redistribute it and/or modify
@@ -18,9 +18,6 @@
    See the COPYING file for a copy of the GNU General Public License.
 */
 
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-
 #include "render.h"
 #include "gradient.h"
 #include "font.h"
@@ -30,6 +27,9 @@
 #include "theme.h"
 
 #include <glib.h>
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/Xft/Xft.h>
 
 #ifdef HAVE_STDLIB_H
 #  include <stdlib.h>
@@ -38,20 +38,27 @@
 static void pixel_data_to_pixmap(RrAppearance *l,
                                  gint x, gint y, gint w, gint h);
 
-void RrPaint(RrAppearance *a, Window win, gint w, gint h)
+Pixmap RrPaintPixmap(RrAppearance *a, gint w, gint h)
 {
-    gint i, transferred = 0, sw, sh, partial_w, partial_h;
-    RrPixel32 *source, *dest;
-    Pixmap oldp;
+    gint i, transferred = 0, force_transfer = 0;
+    Pixmap oldp = None;
     RrRect tarea; /* area in which to draw textures */
     gboolean resized;
 
-    if (w <= 0 || h <= 0) return;
+    if (w <= 0 || h <= 0) return None;
 
     if (a->surface.parentx < 0 || a->surface.parenty < 0) {
         /* ob_debug("Invalid parent co-ordinates\n"); */
-        return;
+        return None;
     }
+
+    if (a->surface.grad == RR_SURFACE_PARENTREL &&
+        (a->surface.parentx >= a->surface.parent->w ||
+         a->surface.parenty >= a->surface.parent->h))
+    {
+        return None;
+    }
+
     resized = (a->w != w || a->h != h);
 
     oldp = a->pixmap; /* save to free after changing the visible pixmap */
@@ -69,37 +76,12 @@ void RrPaint(RrAppearance *a, Window win, gint w, gint h)
                                RrVisual(a->inst), RrColormap(a->inst));
     g_assert(a->xftdraw != NULL);
 
-    g_free(a->surface.pixel_data);
-    a->surface.pixel_data = g_new(RrPixel32, w * h);
+    if (resized) {
+        g_free(a->surface.pixel_data);
+        a->surface.pixel_data = g_new(RrPixel32, w * h);
+    }
 
-    if (a->surface.grad == RR_SURFACE_PARENTREL) {
-        g_assert (a->surface.parent);
-        g_assert (a->surface.parent->w);
-
-        sw = a->surface.parent->w;
-        sh = a->surface.parent->h;
-
-        if (a->surface.parentx >= sw || a->surface.parenty >= sh) {
-            return;
-        }
-
-        source = (a->surface.parent->surface.pixel_data +
-                  a->surface.parentx + sw * a->surface.parenty);
-        dest = a->surface.pixel_data;
-
-        if (a->surface.parentx + w > sw) {
-            partial_w = sw - a->surface.parentx;
-        } else partial_w = w;
-
-        if (a->surface.parenty + h > sh) {
-            partial_h = sh - a->surface.parenty;
-        } else partial_h = h;
-
-        for (i = 0; i < partial_h; i++, source += sw, dest += w) {
-            memcpy(dest, source, partial_w * sizeof(RrPixel32));
-        }
-    } else
-        RrRender(a, w, h);
+    RrRender(a, w, h);
 
     {
         gint l, t, r, b;
@@ -114,7 +96,8 @@ void RrPaint(RrAppearance *a, Window win, gint w, gint h)
         case RR_TEXTURE_TEXT:
             if (!transferred) {
                 transferred = 1;
-                if ((a->surface.grad != RR_SURFACE_SOLID) || (a->surface.interlaced))
+                if ((a->surface.grad != RR_SURFACE_SOLID)
+                    || (a->surface.interlaced))
                     pixel_data_to_pixmap(a, 0, 0, w, h);
             }
             if (a->xftdraw == NULL) {
@@ -127,7 +110,8 @@ void RrPaint(RrAppearance *a, Window win, gint w, gint h)
         case RR_TEXTURE_LINE_ART:
             if (!transferred) {
                 transferred = 1;
-                if ((a->surface.grad != RR_SURFACE_SOLID) || (a->surface.interlaced))
+                if ((a->surface.grad != RR_SURFACE_SOLID)
+                    || (a->surface.interlaced))
                     pixel_data_to_pixmap(a, 0, 0, w, h);
             }
             XDrawLine(RrDisplay(a->inst), a->pixmap,
@@ -140,7 +124,8 @@ void RrPaint(RrAppearance *a, Window win, gint w, gint h)
         case RR_TEXTURE_MASK:
             if (!transferred) {
                 transferred = 1;
-                if ((a->surface.grad != RR_SURFACE_SOLID) || (a->surface.interlaced))
+                if ((a->surface.grad != RR_SURFACE_SOLID)
+                    || (a->surface.interlaced))
                     pixel_data_to_pixmap(a, 0, 0, w, h);
             }
             RrPixmapMaskDraw(a->pixmap, &a->texture[i].data.mask, &tarea);
@@ -151,18 +136,31 @@ void RrPaint(RrAppearance *a, Window win, gint w, gint h)
                         &a->texture[i].data.rgba,
                         a->w, a->h,
                         &tarea);
+            force_transfer = 1;
         break;
         }
     }
 
     if (!transferred) {
         transferred = 1;
-        if ((a->surface.grad != RR_SURFACE_SOLID) || (a->surface.interlaced))
+        if ((a->surface.grad != RR_SURFACE_SOLID) || (a->surface.interlaced) ||
+            force_transfer)
+        {
             pixel_data_to_pixmap(a, 0, 0, w, h);
+        }
     }
 
+    return oldp;
+}
+
+void RrPaint(RrAppearance *a, Window win, gint w, gint h)
+{
+    Pixmap oldp;
+
+    oldp = RrPaintPixmap(a, w, h);
     XSetWindowBackgroundPixmap(RrDisplay(a->inst), win, a->pixmap);
     XClearWindow(RrDisplay(a->inst), win);
+    /* free this after changing the visible pixmap */
     if (oldp) XFreePixmap(RrDisplay(a->inst), oldp);
 }
 
@@ -178,6 +176,14 @@ RrAppearance *RrAppearanceNew(const RrInstance *inst, gint numtex)
   return out;
 }
 
+void RrAppearanceAddTextures(RrAppearance *a, gint numtex)
+{
+    g_assert(a->textures == 0);
+
+    a->textures = numtex;
+    if (numtex) a->texture = g_new0(RrTexture, numtex);
+}
+
 RrAppearance *RrAppearanceCopy(RrAppearance *orig)
 {
     RrSurface *spo, *spc;
@@ -328,49 +334,85 @@ void RrMargins (RrAppearance *a, gint *l, gint *t, gint *r, gint *b)
     }
 }
 
-void RrMinsize(RrAppearance *a, gint *w, gint *h)
+void RrMinSize(RrAppearance *a, gint *w, gint *h)
+{
+    *w = RrMinWidth(a);
+    *h = RrMinHeight(a);
+}
+
+gint RrMinWidth(RrAppearance *a)
 {
     gint i;
     RrSize *m;
     gint l, t, r, b;
-    *w = *h = 0;
+    gint w = 0;
 
     for (i = 0; i < a->textures; ++i) {
         switch (a->texture[i].type) {
         case RR_TEXTURE_NONE:
             break;
         case RR_TEXTURE_MASK:
-            *w = MAX(*w, a->texture[i].data.mask.mask->width);
-            *h = MAX(*h, a->texture[i].data.mask.mask->height);
+            w = MAX(w, a->texture[i].data.mask.mask->width);
             break;
         case RR_TEXTURE_TEXT:
             m = RrFontMeasureString(a->texture[i].data.text.font,
-                                    a->texture[i].data.text.string);
-            *w = MAX(*w, m->width + 4);
-            m->height = RrFontHeight(a->texture[i].data.text.font);
-            *h += MAX(*h, m->height);
+                                    a->texture[i].data.text.string, 
+                                    a->texture[i].data.text.shadow_offset_x,
+                                    a->texture[i].data.text.shadow_offset_y);
+            w = MAX(w, m->width);
             g_free(m);
             break;
         case RR_TEXTURE_RGBA:
-            *w += MAX(*w, a->texture[i].data.rgba.width);
-            *h += MAX(*h, a->texture[i].data.rgba.height);
+            w += MAX(w, a->texture[i].data.rgba.width);
             break;
         case RR_TEXTURE_LINE_ART:
-            *w += MAX(*w, MAX(a->texture[i].data.lineart.x1,
-                              a->texture[i].data.lineart.x2));
-            *h += MAX(*h, MAX(a->texture[i].data.lineart.y1,
-                              a->texture[i].data.lineart.y2));
+            w += MAX(w, MAX(a->texture[i].data.lineart.x1,
+                            a->texture[i].data.lineart.x2));
             break;
         }
     }
 
     RrMargins(a, &l, &t, &r, &b);
 
-    *w += l + r;
-    *h += t + b;
+    w += l + r;
 
-    if (*w < 1) *w = 1;
-    if (*h < 1) *h = 1;
+    if (w < 1) w = 1;
+    return w;
+}
+
+gint RrMinHeight(RrAppearance *a)
+{
+    gint i;
+    gint l, t, r, b;
+    gint h = 0;
+
+    for (i = 0; i < a->textures; ++i) {
+        switch (a->texture[i].type) {
+        case RR_TEXTURE_NONE:
+            break;
+        case RR_TEXTURE_MASK:
+            h = MAX(h, a->texture[i].data.mask.mask->height);
+            break;
+        case RR_TEXTURE_TEXT:
+            h += MAX(h, RrFontHeight(a->texture[i].data.text.font,
+                                     a->texture[i].data.text.shadow_offset_y));
+            break;
+        case RR_TEXTURE_RGBA:
+            h += MAX(h, a->texture[i].data.rgba.height);
+            break;
+        case RR_TEXTURE_LINE_ART:
+            h += MAX(h, MAX(a->texture[i].data.lineart.y1,
+                            a->texture[i].data.lineart.y2));
+            break;
+        }
+    }
+
+    RrMargins(a, &l, &t, &r, &b);
+
+    h += t + b;
+
+    if (h < 1) h = 1;
+    return h;
 }
 
 static void reverse_bits(gchar *c, gint n)
